@@ -7,7 +7,7 @@ import (
 	"strings"
 
 	libconfig "github.com/cri-o/cri-o/pkg/config"
-	"github.com/cri-o/cri-o/server/metrics/collectors"
+	"github.com/cri-o/cri-o/server/otel-collector/collectors"
 	"github.com/sirupsen/logrus"
 	"github.com/urfave/cli/v2"
 )
@@ -362,6 +362,27 @@ func mergeConfig(config *libconfig.Config, ctx *cli.Context) error {
 	if ctx.IsSet("tracing-sampling-rate-per-million") {
 		config.TracingSamplingRatePerMillion = ctx.Int("tracing-sampling-rate-per-million")
 	}
+	if ctx.IsSet("enable-nri") {
+		config.NRI.Enabled = ctx.Bool("enable-nri")
+	}
+	if ctx.IsSet("nri-listen") {
+		config.NRI.SocketPath = ctx.String("nri-listen")
+	}
+	if ctx.IsSet("nri-plugin-dir") {
+		config.NRI.PluginPath = ctx.String("nri-plugin-dir")
+	}
+	if ctx.IsSet("nri-plugin-config-dir") {
+		config.NRI.PluginConfigPath = ctx.String("nri-plugin-config-dir")
+	}
+	if ctx.IsSet("nri-disable-connections") {
+		config.NRI.DisableConnections = ctx.Bool("nri-disable-connections")
+	}
+	if ctx.IsSet("nri-plugin-registration-timeout") {
+		config.NRI.PluginRegistrationTimeout = ctx.Duration("nri-plugin-registration-timeout")
+	}
+	if ctx.IsSet("nri-plugin-request-timeout") {
+		config.NRI.PluginRequestTimeout = ctx.Duration("nri-plugin-request-timeout")
+	}
 	if ctx.IsSet("big-files-temporary-dir") {
 		config.BigFilesTemporaryDir = ctx.String("big-files-temporary-dir")
 	}
@@ -374,7 +395,9 @@ func mergeConfig(config *libconfig.Config, ctx *cli.Context) error {
 	if ctx.IsSet("stats-collection-period") {
 		config.StatsCollectionPeriod = ctx.Int("stats-collection-period")
 	}
-
+	if ctx.IsSet("enable-pod-events") {
+		config.EnablePodEvents = ctx.Bool("enable-pod-events")
+	}
 	return nil
 }
 
@@ -805,7 +828,7 @@ func getCrioFlags(defConf *libconfig.Config) []cli.Flag {
 		&cli.IntFlag{
 			Name:    "tracing-sampling-rate-per-million",
 			Value:   defConf.TracingSamplingRatePerMillion,
-			Usage:   "Number of samples to collect per million OpenTelemetry spans.",
+			Usage:   "Number of samples to collect per million OpenTelemetry spans. Set to 1000000 to always sample.",
 			EnvVars: []string{"CONTAINER_TRACING_SAMPLING_RATE_PER_MILLION"},
 		},
 		&cli.StringFlag{
@@ -814,6 +837,36 @@ func getCrioFlags(defConf *libconfig.Config) []cli.Flag {
 			Usage:   "Address on which the gRPC tracing collector will listen.",
 			EnvVars: []string{"CONTAINER_TRACING_ENDPOINT"},
 		},
+		&cli.BoolFlag{
+			Name:  "enable-nri",
+			Usage: fmt.Sprintf("Enable NRI (Node Resource Interface) support. (default: %v)", defConf.NRI.Enabled),
+		},
+		&cli.StringFlag{
+			Name:  "nri-listen",
+			Usage: fmt.Sprintf("Socket to listen on for externally started NRI plugins to connect to. (default: %q)", defConf.NRI.SocketPath),
+		},
+		&cli.StringFlag{
+			Name:  "nri-plugin-dir",
+			Usage: fmt.Sprintf("Directory to scan for pre-installed NRI plugins to start automatically. (default: %q)", defConf.NRI.PluginPath),
+		},
+		&cli.StringFlag{
+			Name:  "nri-plugin-config-dir",
+			Usage: fmt.Sprintf("Directory to scan for configuration of pre-installed NRI plugins. (default: %q)", defConf.NRI.PluginConfigPath),
+		},
+		&cli.StringFlag{
+			Name:  "nri-disable-connections",
+			Usage: fmt.Sprintf("Disable connections from externally started NRI plugins. (default: %v)", defConf.NRI.DisableConnections),
+		},
+		&cli.DurationFlag{
+			Name:  "nri-plugin-registration-timeout",
+			Usage: `Timeout for a plugin to register itself with NRI.`,
+			Value: defConf.NRI.PluginRegistrationTimeout,
+		},
+		&cli.DurationFlag{
+			Name:  "nri-plugin-request-timeout",
+			Usage: `Timeout for a plugin to handle an NRI request.`,
+			Value: defConf.NRI.PluginRequestTimeout,
+		},
 		&cli.StringFlag{
 			Name:    "big-files-temporary-dir",
 			Usage:   `Path to the temporary directory to use for storing big files, used to store image blobs and data streams related to containers image management.`,
@@ -1048,6 +1101,11 @@ func getCrioFlags(defConf *libconfig.Config) []cli.Flag {
 			EnvVars: []string{"CONTAINER_ENABLE_CRIU_SUPPORT"},
 			Value:   false,
 		},
+		&cli.BoolFlag{
+			Name:    "enable-pod-events",
+			Usage:   "If true, CRI-O starts sending the container events to the kubelet",
+			EnvVars: []string{"ENABLE_POD_EVENTS"},
+		},
 	}
 }
 
