@@ -26,16 +26,17 @@ import (
 	"strings"
 	"sync"
 
+	"github.com/sirupsen/logrus"
+	"github.com/vishvananda/netlink"
+	"golang.org/x/sys/unix"
 	v1 "k8s.io/api/core/v1"
 	utilerrors "k8s.io/apimachinery/pkg/util/errors"
-	"k8s.io/klog/v2"
-	iptablesproxy "k8s.io/kubernetes/pkg/proxy/iptables"
-	"k8s.io/kubernetes/pkg/util/conntrack"
 	utiliptables "k8s.io/kubernetes/pkg/util/iptables"
-	"k8s.io/utils/exec"
 	utilnet "k8s.io/utils/net"
 )
 
+const kubeMarkMasqChain = "KUBE-MARK-MASQ"
+
 // HostPortManager is an interface for adding and removing hostport for a given pod sandbox.
 // nolint:golint // no reason to change the type name now "type name will be used as hostport.HostPortManager by other packages"
 type HostPortManager interface {
@@ -50,26 +51,20 @@ type HostPortManager interface {
 }
 
 type hostportManager struct {
-	hostPortMap    map[hostport]closeable
-	execer         exec.Interface
-	conntrackFound bool
-	iptables       utiliptables.Interface
-	portOpener     hostportOpener
-	mu             sync.Mutex
+	hostPortMap map[hostport]closeable
+	iptables    utiliptables.Interface
+	portOpener  hostportOpener
+	mu          sync.Mutex
 }
 
 // NewHostportManager creates a new HostPortManager
 func NewHostportManager(iptables utiliptables.Interface) HostPortManager {
 	h := &hostportManager{
 		hostPortMap: make(map[hostport]closeable),
-		execer:      exec.New(),
 		iptables:    iptables,
 		portOpener:  openLocalPort,
 	}
-	h.conntrackFound = conntrack.Exists(h.execer)
-	if !h.conntrackFound {
-		klog.Warningf("The binary conntrack is not installed, this can cause failures in network connection cleanup.")
-	}
+
 	return h
 }
 
@@ -147,7 +142,7 @@ func (hm *hostportManager) Add(id string, podPortMapping *PodPortMapping, natInt
 		writeLine(natRules, "-A", string(chain),
 			"-m", "comment", "--comment", fmt.Sprintf(`"%s hostport %d"`, podFullName, pm.HostPort),
 			"-s", podIP,
-			"-j", string(iptablesproxy.KubeMarkMasqChain))
+			"-j", kubeMarkMasqChain)
 
 		// DNAT to the podIP:containerPort
 		hostPortBinding := net.JoinHostPort(podIP, strconv.Itoa(int(pm.ContainerPort)))
@@ -187,13 +182,13 @@ func (hm *hostportManager) Add(id string, podPortMapping *PodPortMapping, natInt
 	// the IP tables rule, it can be the case that the packets received by the node after iptables rule removal will
 	// create a new conntrack entry without any DNAT. That will result in blackhole of the traffic even after correct
 	// iptables rules have been added back.
-	if hm.execer != nil && hm.conntrackFound {
-		klog.Infof("Starting to delete udp conntrack entries: %v, isIPv6 - %v", conntrackPortsToRemove, isIPv6)
-		for _, port := range conntrackPortsToRemove {
-			err = conntrack.ClearEntriesForPort(hm.execer, port, isIPv6, v1.ProtocolUDP)
-			if err != nil {
-				klog.Errorf("Failed to clear udp conntrack for port %d, error: %v", port, err)
-			}
+	logrus.Infof("Starting to delete udp conntrack entries: %v, isIPv6 - %v", conntrackPortsToRemove, isIPv6)
+	// https://www.iana.org/assignments/protocol-numbers/protocol-numbers.xhtml
+	const protocolUDPNumber = 17
+	for _, port := range conntrackPortsToRemove {
+		err = deleteConntrackEntriesForDstPort(uint16(port), protocolUDPNumber, getNetlinkFamily(isIPv6))
+		if err != nil {
+			logrus.Errorf("Failed to clear udp conntrack for port %d, error: %v", port, err)
 		}
 	}
 	return nil
@@ -267,10 +262,10 @@ func (hm *hostportManager) Remove(id string, podPortMapping *PodPortMapping) (er
 
 // syncIPTables executes iptables-restore with given lines
 func (hm *hostportManager) syncIPTables(lines []byte) error {
-	klog.V(3).Infof("Restoring iptables rules: %s", lines)
+	logrus.Infof("Restoring iptables rules: %s", lines)
 	err := hm.iptables.RestoreAll(lines, utiliptables.NoFlushTables, utiliptables.RestoreCounters)
 	if err != nil {
-		return fmt.Errorf("failed to execute iptables-restore: %v", err)
+		return fmt.Errorf("failed to execute iptables-restore: %w", err)
 	}
 	return nil
 }
@@ -299,7 +294,7 @@ func (hm *hostportManager) openHostports(podPortMapping *PodPortMapping) (map[ho
 		hp := portMappingToHostport(pm, hm.getIPFamily())
 		socket, err := hm.portOpener(&hp)
 		if err != nil {
-			retErr = fmt.Errorf("cannot open hostport %d for pod %s: %v", pm.HostPort, getPodFullName(podPortMapping), err)
+			retErr = fmt.Errorf("cannot open hostport %d for pod %s: %w", pm.HostPort, getPodFullName(podPortMapping), err)
 			break
 		}
 		ports[hp] = socket
@@ -309,7 +304,7 @@ func (hm *hostportManager) openHostports(podPortMapping *PodPortMapping) (map[ho
 	if retErr != nil {
 		for hp, socket := range ports {
 			if err := socket.Close(); err != nil {
-				klog.Errorf("Cannot clean up hostport %d for pod %s: %v", hp.port, getPodFullName(podPortMapping), err)
+				logrus.Errorf("Cannot clean up hostport %d for pod %s: %v", hp.port, getPodFullName(podPortMapping), err)
 			}
 		}
 		return nil, retErr
@@ -323,14 +318,14 @@ func (hm *hostportManager) closeHostports(hostportMappings []*PortMapping) error
 	for _, pm := range hostportMappings {
 		hp := portMappingToHostport(pm, hm.getIPFamily())
 		if socket, ok := hm.hostPortMap[hp]; ok {
-			klog.V(2).Infof("Closing host port %s", hp.String())
+			logrus.Infof("Closing host port %s", hp.String())
 			if err := socket.Close(); err != nil {
-				errList = append(errList, fmt.Errorf("failed to close host port %s: %v", hp.String(), err))
+				errList = append(errList, fmt.Errorf("failed to close host port %s: %w", hp.String(), err))
 				continue
 			}
 			delete(hm.hostPortMap, hp)
 		} else {
-			klog.V(5).Infof("host port %s does not have an open socket", hp.String())
+			logrus.Infof("Host port %s does not have an open socket", hp.String())
 		}
 	}
 	return utilerrors.NewAggregate(errList)
@@ -380,9 +375,9 @@ func getExistingHostportIPTablesRules(iptables utiliptables.Interface) (map[util
 	iptablesData := bytes.NewBuffer(nil)
 	err := iptables.SaveInto(utiliptables.TableNAT, iptablesData)
 	if err != nil { // if we failed to get any rules
-		return nil, nil, fmt.Errorf("failed to execute iptables-save: %v", err)
+		return nil, nil, fmt.Errorf("failed to execute iptables-save: %w", err)
 	}
-	existingNATChains := utiliptables.GetChainLines(utiliptables.TableNAT, iptablesData.Bytes())
+	existingNATChains := getChainLines(utiliptables.TableNAT, iptablesData.Bytes())
 
 	existingHostportChains := make(map[utiliptables.Chain]string)
 	existingHostportRules := []string{}
@@ -402,6 +397,100 @@ func getExistingHostportIPTablesRules(iptables utiliptables.Interface) (map[util
 	return existingHostportChains, existingHostportRules, nil
 }
 
+// getChainLines parses a table's iptables-save data to find chains in the table.
+// It returns a map of iptables.Chain to []byte where the []byte is the chain line
+// from save (with counters etc.).
+// Note that to avoid allocations memory is SHARED with save.
+func getChainLines(table utiliptables.Table, save []byte) map[utiliptables.Chain][]byte {
+	chainsMap := make(map[utiliptables.Chain][]byte)
+	tablePrefix := []byte("*" + string(table))
+	readIndex := 0
+	// find beginning of table
+	for readIndex < len(save) {
+		line, n := readLine(readIndex, save)
+		readIndex = n
+		if bytes.HasPrefix(line, tablePrefix) {
+			break
+		}
+	}
+	var (
+		commitBytes = []byte("COMMIT")
+		spaceBytes  = []byte(" ")
+	)
+	// parse table lines
+	for readIndex < len(save) {
+		line, n := readLine(readIndex, save)
+		readIndex = n
+		if len(line) == 0 {
+			continue
+		}
+		if bytes.HasPrefix(line, commitBytes) || line[0] == '*' { //nolint:gocritic
+			break
+		} else if line[0] == '#' {
+			continue
+		} else if line[0] == ':' && len(line) > 1 {
+			// We assume that the <line> contains space - chain lines have 3 fields,
+			// space delimited. If there is no space, this line will panic.
+			spaceIndex := bytes.Index(line, spaceBytes)
+			if spaceIndex == -1 {
+				panic(fmt.Sprintf("Unexpected chain line in iptables-save output: %v", string(line)))
+			}
+			chain := utiliptables.Chain(line[1:spaceIndex])
+			chainsMap[chain] = line
+		}
+	}
+	return chainsMap
+}
+
+func readLine(readIndex int, byteArray []byte) (line []byte, n int) {
+	currentReadIndex := readIndex
+
+	// consume left spaces
+	for currentReadIndex < len(byteArray) {
+		if byteArray[currentReadIndex] == ' ' {
+			currentReadIndex++
+		} else {
+			break
+		}
+	}
+
+	// leftTrimIndex stores the left index of the line after the line is left-trimmed
+	leftTrimIndex := currentReadIndex
+
+	// rightTrimIndex stores the right index of the line after the line is right-trimmed
+	// it is set to -1 since the correct value has not yet been determined.
+	rightTrimIndex := -1
+
+	for ; currentReadIndex < len(byteArray); currentReadIndex++ {
+		if byteArray[currentReadIndex] == ' ' { //nolint:gocritic
+			// set rightTrimIndex
+			if rightTrimIndex == -1 {
+				rightTrimIndex = currentReadIndex
+			}
+		} else if (byteArray[currentReadIndex] == '\n') || (currentReadIndex == (len(byteArray) - 1)) {
+			// end of line or byte buffer is reached
+			if currentReadIndex <= leftTrimIndex {
+				return nil, currentReadIndex + 1
+			}
+			// set the rightTrimIndex
+			if rightTrimIndex == -1 {
+				rightTrimIndex = currentReadIndex
+				if currentReadIndex == (len(byteArray)-1) && (byteArray[currentReadIndex] != '\n') {
+					// ensure that the last character is part of the returned string,
+					// unless the last character is '\n'
+					rightTrimIndex = currentReadIndex + 1
+				}
+			}
+			// Avoid unnecessary allocation.
+			return byteArray[leftTrimIndex:rightTrimIndex], currentReadIndex + 1
+		} else {
+			// unset rightTrimIndex
+			rightTrimIndex = -1
+		}
+	}
+	return nil, currentReadIndex
+}
+
 // filterRules filters input rules with input chains. Rules that did not involve any filter chain will be returned.
 // The order of the input rules is important and is preserved.
 func filterRules(rules []string, filters []utiliptables.Chain) []string {
@@ -443,3 +532,30 @@ func writeLine(buf *bytes.Buffer, words ...string) {
 func (hp *hostport) String() string {
 	return fmt.Sprintf("%s:%d", hp.protocol, hp.port)
 }
+
+// deleteConntrackEntriesForDstPort delete the conntrack entries for the connections specified
+// by the given destination port, protocol and IP family
+func deleteConntrackEntriesForDstPort(port uint16, protocol uint8, family netlink.InetFamily) error {
+	filter := &netlink.ConntrackFilter{}
+	err := filter.AddProtocol(protocol)
+	if err != nil {
+		return fmt.Errorf("error deleting connection tracking state for protocol: %d Port: %d, error: %w", protocol, port, err)
+	}
+	err = filter.AddPort(netlink.ConntrackOrigDstPort, port)
+	if err != nil {
+		return fmt.Errorf("error deleting connection tracking state for protocol: %d Port: %d, error: %w", protocol, port, err)
+	}
+
+	_, err = netlink.ConntrackDeleteFilter(netlink.ConntrackTable, family, filter)
+	if err != nil {
+		return fmt.Errorf("error deleting connection tracking state for protocol: %d Port: %d, error: %w", protocol, port, err)
+	}
+	return nil
+}
+
+func getNetlinkFamily(isIPv6 bool) netlink.InetFamily {
+	if isIPv6 {
+		return unix.AF_INET6
+	}
+	return unix.AF_INET
+}
