@@ -1,11 +1,10 @@
+//go:build !linux
 // +build !linux
 
 package lib
 
 import (
 	"github.com/cri-o/cri-o/internal/lib/sandbox"
-	"github.com/cri-o/cri-o/internal/oci"
-	"github.com/pkg/errors"
 )
 
 func (c *ContainerServer) addSandboxPlatform(sb *sandbox.Sandbox) {
