@@ -3,8 +3,10 @@ package log
 
 import (
 	"context"
+	"runtime"
 
 	"github.com/sirupsen/logrus"
+	"go.opentelemetry.io/otel/trace"
 )
 
 type (
@@ -32,6 +34,10 @@ func Fatalf(ctx context.Context, format string, args ...interface{}) {
 	entry(ctx).Fatalf(format, args...)
 }
 
+func WithFields(ctx context.Context, fields map[string]interface{}) *logrus.Entry {
+	return entry(ctx).WithFields(fields)
+}
+
 func entry(ctx context.Context) *logrus.Entry {
 	logger := logrus.StandardLogger()
 	if ctx == nil {
@@ -41,8 +47,19 @@ func entry(ctx context.Context) *logrus.Entry {
 	id, idOk := ctx.Value(ID{}).(string)
 	name, nameOk := ctx.Value(Name{}).(string)
 	if idOk && nameOk {
-		return logger.WithField("id", id).WithField("name", name)
+		return logger.WithField("id", id).WithField("name", name).WithContext(ctx)
 	}
 
-	return logrus.NewEntry(logger)
+	return logrus.NewEntry(logger).WithContext(ctx)
+}
+
+func StartSpan(ctx context.Context) (context.Context, trace.Span) {
+	spanName := "unknown"
+	// Use function signature as a span name if available
+	if pc, _, _, ok := runtime.Caller(1); ok {
+		spanName = runtime.FuncForPC(pc).Name()
+	} else {
+		Debugf(ctx, "Unable to retrieve a caller when starting span")
+	}
+	return trace.SpanFromContext(ctx).TracerProvider().Tracer("").Start(ctx, spanName)
 }
