@@ -2,8 +2,8 @@ package oci
 
 import (
 	"bytes"
+	"errors"
 	"fmt"
-	"io/ioutil"
 	"os"
 	"path/filepath"
 	"strconv"
@@ -12,15 +12,17 @@ import (
 	"syscall"
 	"time"
 
+	metadata "github.com/checkpoint-restore/checkpointctl/lib"
+	"github.com/containers/common/pkg/cgroups"
 	"github.com/containers/common/pkg/signal"
-	"github.com/containers/podman/v3/pkg/cgroups"
 	"github.com/containers/storage/pkg/idtools"
 	"github.com/cri-o/cri-o/internal/config/nsmgr"
+	"github.com/cri-o/cri-o/internal/log"
 	ann "github.com/cri-o/cri-o/pkg/annotations"
 	json "github.com/json-iterator/go"
 	specs "github.com/opencontainers/runtime-spec/specs-go"
-	"github.com/pkg/errors"
 	"github.com/sirupsen/logrus"
+	"golang.org/x/net/context"
 	"golang.org/x/sys/unix"
 	"k8s.io/apimachinery/pkg/fields"
 	types "k8s.io/cri-api/pkg/apis/runtime/v1"
@@ -74,6 +76,9 @@ type Container struct {
 	stopStoppingChan   chan struct{}
 	stopLock           sync.Mutex
 	pidns              nsmgr.Namespace
+	restore            bool
+	restoreArchive     string
+	restoreIsOCIImage  bool
 }
 
 func (c *Container) CRIAttributes() *types.ContainerAttributes {
@@ -97,13 +102,14 @@ type ContainerVolume struct {
 // ContainerState represents the status of a container.
 type ContainerState struct {
 	specs.State
-	Created   time.Time `json:"created"`
-	Started   time.Time `json:"started,omitempty"`
-	Finished  time.Time `json:"finished,omitempty"`
-	ExitCode  *int32    `json:"exitCode,omitempty"`
-	OOMKilled bool      `json:"oomKilled,omitempty"`
-	Error     string    `json:"error,omitempty"`
-	InitPid   int       `json:"initPid,omitempty"`
+	Created       time.Time `json:"created"`
+	Started       time.Time `json:"started,omitempty"`
+	Finished      time.Time `json:"finished,omitempty"`
+	ExitCode      *int32    `json:"exitCode,omitempty"`
+	OOMKilled     bool      `json:"oomKilled,omitempty"`
+	SeccompKilled bool      `json:"seccompKilled,omitempty"`
+	Error         string    `json:"error,omitempty"`
+	InitPid       int       `json:"initPid,omitempty"`
 	// The unix start time of the container's init PID.
 	// This is used to track whether the PID we have stored
 	// is the same as the corresponding PID on the host.
@@ -111,7 +117,7 @@ type ContainerState struct {
 }
 
 // NewContainer creates a container object.
-func NewContainer(id, name, bundlePath, logPath string, labels, crioAnnotations, annotations map[string]string, image, imageName, imageRef string, metadata *types.ContainerMetadata, sandbox string, terminal, stdin, stdinOnce bool, runtimeHandler, dir string, created time.Time, stopSignal string) (*Container, error) {
+func NewContainer(id, name, bundlePath, logPath string, labels, crioAnnotations, annotations map[string]string, image, imageName, imageRef string, md *types.ContainerMetadata, sandbox string, terminal, stdin, stdinOnce bool, runtimeHandler, dir string, created time.Time, stopSignal string) (*Container, error) {
 	state := &ContainerState{}
 	state.Created = created
 	c := &Container{
@@ -120,7 +126,7 @@ func NewContainer(id, name, bundlePath, logPath string, labels, crioAnnotations,
 			PodSandboxId: sandbox,
 			CreatedAt:    created.UnixNano(),
 			Labels:       labels,
-			Metadata:     metadata,
+			Metadata:     md,
 			Annotations:  annotations,
 			Image: &types.ImageSpec{
 				Image: image,
@@ -171,10 +177,24 @@ func NewSpoofedContainer(id, name string, labels map[string]string, sandbox stri
 }
 
 func (c *Container) CRIContainer() *types.Container {
-	// Return a deep copy so the State field doesn't get mutated mid-request,
-	// causing a proto panic.
-	cpy := *c.criContainer
-	return &cpy
+	// If a protobuf message gets mutated mid-request, then the proto library panics.
+	// We would like to avoid deep copies when possible to avoid excessive garbage
+	// collection, but need to if the container changes state.
+	newState := types.ContainerState_CONTAINER_UNKNOWN
+	switch c.StateNoLock().Status {
+	case ContainerStateCreated:
+		newState = types.ContainerState_CONTAINER_CREATED
+	case ContainerStateRunning, ContainerStatePaused:
+		newState = types.ContainerState_CONTAINER_RUNNING
+	case ContainerStateStopped:
+		newState = types.ContainerState_CONTAINER_EXITED
+	}
+	if newState != c.criContainer.State {
+		cpy := *c.criContainer
+		cpy.State = newState
+		c.criContainer = &cpy
+	}
+	return c.criContainer
 }
 
 // SetSpec loads the OCI spec in the container struct
@@ -184,7 +204,10 @@ func (c *Container) SetSpec(s *specs.Spec) {
 
 // Spec returns a copy of the spec for the container
 func (c *Container) Spec() specs.Spec {
-	return *c.spec
+	if c.spec != nil {
+		return *c.spec
+	}
+	return specs.Spec{}
 }
 
 // ConmonCgroupfsPath returns the path to conmon's cgroup. This is only set when
@@ -252,7 +275,7 @@ func (c *Container) FromDisk() error {
 // These values should be set once, and not changed again.
 func (cstate *ContainerState) SetInitPid(pid int) error {
 	if cstate.InitPid != 0 || cstate.InitStartTime != "" {
-		return errors.Errorf("pid and start time already initialized: %d %s", cstate.InitPid, cstate.InitStartTime)
+		return fmt.Errorf("pid and start time already initialized: %d %s", cstate.InitPid, cstate.InitStartTime)
 	}
 	cstate.InitPid = pid
 	startTime, err := getPidStartTime(pid)
@@ -284,7 +307,9 @@ func (c *Container) ID() string {
 }
 
 // CleanupConmonCgroup cleans up conmon's group when using cgroupfs.
-func (c *Container) CleanupConmonCgroup() {
+func (c *Container) CleanupConmonCgroup(ctx context.Context) {
+	ctx, span := log.StartSpan(ctx)
+	defer span.End()
 	if c.spoofed {
 		return
 	}
@@ -294,11 +319,11 @@ func (c *Container) CleanupConmonCgroup() {
 	}
 	cg, err := cgroups.Load(path)
 	if err != nil {
-		logrus.Infof("Error loading conmon cgroup of container %s: %v", c.ID(), err)
+		log.Infof(ctx, "Error loading conmon cgroup of container %s: %v", c.ID(), err)
 		return
 	}
 	if err := cg.Delete(); err != nil {
-		logrus.Infof("Error deleting conmon cgroup of container %s: %v", c.ID(), err)
+		log.Infof(ctx, "Error deleting conmon cgroup of container %s: %v", c.ID(), err)
 	}
 }
 
@@ -357,11 +382,27 @@ func (c *Container) Sandbox() string {
 	return c.criContainer.PodSandboxId
 }
 
+// SetSandbox sets the ID of the Sandbox.
+func (c *Container) SetSandbox(podSandboxID string) {
+	c.criContainer.PodSandboxId = podSandboxID
+}
+
 // Dir returns the dir of the container
 func (c *Container) Dir() string {
 	return c.dir
 }
 
+// CheckpointPath returns the path to the directory containing the checkpoint
+func (c *Container) CheckpointPath() string {
+	// Podman uses 'bundlePath' as base directory for the checkpoint
+	// CRI-O uses 'dir' instead of bundlePath as bundlePath seems to be
+	// normally based on a tmpfs which does not survive a reboot. Also, as
+	// the checkpoint contains all memory pages, it can be as large as the
+	// available memory and writing that again to a tmpfs might lead to
+	// problems. 'dir' seems to be based on /var
+	return filepath.Join(c.dir, metadata.CheckpointDirectory)
+}
+
 // Metadata returns the metadata of the container.
 func (c *Container) Metadata() *types.ContainerMetadata {
 	return c.criContainer.Metadata
@@ -447,8 +488,10 @@ func (c *Container) exitFilePath() string {
 // IsAlive is a function that checks if a container's init PID exists.
 // It is used to check a container state when we don't want a `$runtime state` call
 func (c *Container) IsAlive() error {
-	_, err := c.pid()
-	return errors.Wrapf(err, "checking if PID of %s is running failed", c.ID())
+	if _, err := c.pid(); err != nil {
+		return fmt.Errorf("checking if PID of %s is running failed: %w", c.ID(), err)
+	}
+	return nil
 }
 
 // Pid returns the container's init PID.
@@ -479,7 +522,7 @@ func (c *Container) pid() (int, error) {
 		return 0, err
 	}
 	if err := unix.Kill(c.state.InitPid, 0); err == unix.ESRCH {
-		return 0, errors.Wrapf(err, "check whether %d is running", c.state.InitPid)
+		return 0, fmt.Errorf("check whether %d is running: %w", c.state.InitPid, err)
 	}
 	return c.state.InitPid, nil
 }
@@ -495,7 +538,7 @@ func (c *Container) verifyPid() error {
 	}
 
 	if startTime != c.state.InitStartTime {
-		return errors.Errorf(
+		return fmt.Errorf(
 			"PID %d is running but has start time of %s, whereas the saved start time is %s. PID wrap may have occurred",
 			c.state.InitPid, startTime, c.state.InitStartTime,
 		)
@@ -513,15 +556,15 @@ func getPidStartTime(pid int) (string, error) {
 // GetPidStartTime reads a file as if it were a /proc/$pid/stat file, looking for stime for PID.
 // It is abstracted out to allow for unit testing
 func GetPidStartTimeFromFile(file string) (string, error) {
-	data, err := ioutil.ReadFile(file)
+	data, err := os.ReadFile(file)
 	if err != nil {
-		return "", errors.Wrapf(ErrNotFound, err.Error())
+		return "", fmt.Errorf("%v: %w", err, ErrNotFound)
 	}
 	// The command (2nd field) can have spaces, but is wrapped in ()
 	// first, trim it
 	commEnd := bytes.LastIndexByte(data, ')')
 	if commEnd == -1 {
-		return "", errors.Wrapf(ErrNotFound, "unable to find ')' in stat file")
+		return "", fmt.Errorf("unable to find ')' in stat file: %w", ErrNotFound)
 	}
 
 	// start on the space after the command
@@ -533,7 +576,7 @@ func GetPidStartTimeFromFile(file string) (string, error) {
 		// find the next space
 		iter = bytes.IndexByte(data, ' ')
 		if iter == -1 {
-			return "", errors.Wrapf(ErrNotFound, "invalid number of entries found in stat file %s: %d", file, field-1)
+			return "", fmt.Errorf("invalid number of entries found in stat file %s: %d: %w", file, field-1, ErrNotFound)
 		}
 	}
 
@@ -609,7 +652,7 @@ func (c *Container) RemoveManagedPIDNamespace() error {
 	if c.pidns == nil {
 		return nil
 	}
-	return errors.Wrapf(c.pidns.Remove(), "remove PID namespace for container %s", c.ID())
+	return fmt.Errorf("remove PID namespace for container %s: %w", c.ID(), c.pidns.Remove())
 }
 
 func (c *Container) IsInfra() bool {
@@ -628,3 +671,30 @@ func (c *Container) nodeLevelPIDNamespace() bool {
 	}
 	return true
 }
+
+// Restore returns if the container is marked as being
+// restored from a checkpoint
+func (c *Container) Restore() bool {
+	return c.restore
+}
+
+// SetRestore marks the container as being restored from a checkpoint
+func (c *Container) SetRestore(restore bool) {
+	c.restore = restore
+}
+
+func (c *Container) RestoreArchive() string {
+	return c.restoreArchive
+}
+
+func (c *Container) SetRestoreArchive(restoreArchive string) {
+	c.restoreArchive = restoreArchive
+}
+
+func (c *Container) RestoreIsOCIImage() bool {
+	return c.restoreIsOCIImage
+}
+
+func (c *Container) SetRestoreIsOCIImage(restoreIsOCIImage bool) {
+	c.restoreIsOCIImage = restoreIsOCIImage
+}
