@@ -6,14 +6,13 @@ import (
 	"io"
 	"os"
 	"path/filepath"
-	"strings"
 	"sync"
 	"syscall"
 	"time"
 
+	"github.com/cri-o/cri-o/internal/log"
 	"github.com/cri-o/cri-o/pkg/config"
 	rspec "github.com/opencontainers/runtime-spec/specs-go"
-	"github.com/pkg/errors"
 	"golang.org/x/net/context"
 	types "k8s.io/cri-api/pkg/apis/runtime/v1"
 
@@ -58,7 +57,7 @@ type Runtime struct {
 // runtimes. Assumptions based on the fact that a container process runs
 // on the host will be limited to the RuntimeOCI implementation.
 type RuntimeImpl interface {
-	CreateContainer(context.Context, *Container, string) error
+	CreateContainer(context.Context, *Container, string, bool) error
 	StartContainer(context.Context, *Container) error
 	ExecContainer(context.Context, *Container, []string, io.Reader, io.WriteCloser, io.WriteCloser,
 		bool, <-chan remotecommand.TerminalSize) error
@@ -76,13 +75,15 @@ type RuntimeImpl interface {
 	PortForwardContainer(context.Context, *Container, string,
 		int32, io.ReadWriteCloser) error
 	ReopenContainerLog(context.Context, *Container) error
+	CheckpointContainer(context.Context, *Container, *rspec.Spec, bool) error
+	RestoreContainer(context.Context, *Container, string, string) error
 }
 
 // New creates a new Runtime with options provided
 func New(c *config.Config) (*Runtime, error) {
 	execNotifyDir := filepath.Join(c.ContainerAttachSocketDir, "exec-pid-dir")
 	if err := os.MkdirAll(execNotifyDir, 0o750); err != nil {
-		return nil, errors.Wrapf(err, "create oci runtime pid dir")
+		return nil, fmt.Errorf("create oci runtime pid dir: %w", err)
 	}
 
 	return &Runtime{
@@ -207,7 +208,9 @@ func (r *Runtime) RuntimeImpl(c *Container) (RuntimeImpl, error) {
 }
 
 // CreateContainer creates a container.
-func (r *Runtime) CreateContainer(ctx context.Context, c *Container, cgroupParent string) error {
+func (r *Runtime) CreateContainer(ctx context.Context, c *Container, cgroupParent string, restore bool) error {
+	ctx, span := log.StartSpan(ctx)
+	defer span.End()
 	// Instantiate a new runtime implementation for this new container
 	impl, err := r.newRuntimeImpl(c)
 	if err != nil {
@@ -219,11 +222,13 @@ func (r *Runtime) CreateContainer(ctx context.Context, c *Container, cgroupParen
 	r.runtimeImplMap[c.ID()] = impl
 	r.runtimeImplMapMutex.Unlock()
 
-	return impl.CreateContainer(ctx, c, cgroupParent)
+	return impl.CreateContainer(ctx, c, cgroupParent, restore)
 }
 
 // StartContainer starts a container.
 func (r *Runtime) StartContainer(ctx context.Context, c *Container) error {
+	ctx, span := log.StartSpan(ctx)
+	defer span.End()
 	impl, err := r.RuntimeImpl(c)
 	if err != nil {
 		return err
@@ -234,6 +239,8 @@ func (r *Runtime) StartContainer(ctx context.Context, c *Container) error {
 
 // ExecContainer prepares a streaming endpoint to execute a command in the container.
 func (r *Runtime) ExecContainer(ctx context.Context, c *Container, cmd []string, stdin io.Reader, stdout, stderr io.WriteCloser, tty bool, resize <-chan remotecommand.TerminalSize) error {
+	ctx, span := log.StartSpan(ctx)
+	defer span.End()
 	impl, err := r.RuntimeImpl(c)
 	if err != nil {
 		return err
@@ -244,6 +251,8 @@ func (r *Runtime) ExecContainer(ctx context.Context, c *Container, cmd []string,
 
 // ExecSyncContainer execs a command in a container and returns it's stdout, stderr and return code.
 func (r *Runtime) ExecSyncContainer(ctx context.Context, c *Container, command []string, timeout int64) (*types.ExecSyncResponse, error) {
+	ctx, span := log.StartSpan(ctx)
+	defer span.End()
 	impl, err := r.RuntimeImpl(c)
 	if err != nil {
 		return nil, err
@@ -254,6 +263,8 @@ func (r *Runtime) ExecSyncContainer(ctx context.Context, c *Container, command [
 
 // UpdateContainer updates container resources
 func (r *Runtime) UpdateContainer(ctx context.Context, c *Container, res *rspec.LinuxResources) error {
+	ctx, span := log.StartSpan(ctx)
+	defer span.End()
 	impl, err := r.RuntimeImpl(c)
 	if err != nil {
 		return err
@@ -264,6 +275,8 @@ func (r *Runtime) UpdateContainer(ctx context.Context, c *Container, res *rspec.
 
 // StopContainer stops a container. Timeout is given in seconds.
 func (r *Runtime) StopContainer(ctx context.Context, c *Container, timeout int64) error {
+	ctx, span := log.StartSpan(ctx)
+	defer span.End()
 	impl, err := r.RuntimeImpl(c)
 	if err != nil {
 		return err
@@ -274,6 +287,8 @@ func (r *Runtime) StopContainer(ctx context.Context, c *Container, timeout int64
 
 // DeleteContainer deletes a container.
 func (r *Runtime) DeleteContainer(ctx context.Context, c *Container) (err error) {
+	ctx, span := log.StartSpan(ctx)
+	defer span.End()
 	r.runtimeImplMapMutex.RLock()
 	impl, ok := r.runtimeImplMap[c.ID()]
 	r.runtimeImplMapMutex.RUnlock()
@@ -296,6 +311,8 @@ func (r *Runtime) DeleteContainer(ctx context.Context, c *Container) (err error)
 
 // UpdateContainerStatus refreshes the status of the container.
 func (r *Runtime) UpdateContainerStatus(ctx context.Context, c *Container) error {
+	ctx, span := log.StartSpan(ctx)
+	defer span.End()
 	impl, err := r.RuntimeImpl(c)
 	if err != nil {
 		return err
@@ -306,6 +323,8 @@ func (r *Runtime) UpdateContainerStatus(ctx context.Context, c *Container) error
 
 // PauseContainer pauses a container.
 func (r *Runtime) PauseContainer(ctx context.Context, c *Container) error {
+	ctx, span := log.StartSpan(ctx)
+	defer span.End()
 	impl, err := r.RuntimeImpl(c)
 	if err != nil {
 		return err
@@ -316,6 +335,8 @@ func (r *Runtime) PauseContainer(ctx context.Context, c *Container) error {
 
 // UnpauseContainer unpauses a container.
 func (r *Runtime) UnpauseContainer(ctx context.Context, c *Container) error {
+	ctx, span := log.StartSpan(ctx)
+	defer span.End()
 	impl, err := r.RuntimeImpl(c)
 	if err != nil {
 		return err
@@ -326,6 +347,8 @@ func (r *Runtime) UnpauseContainer(ctx context.Context, c *Container) error {
 
 // ContainerStats provides statistics of a container.
 func (r *Runtime) ContainerStats(ctx context.Context, c *Container, cgroup string) (*types.ContainerStats, error) {
+	ctx, span := log.StartSpan(ctx)
+	defer span.End()
 	impl, err := r.RuntimeImpl(c)
 	if err != nil {
 		return nil, err
@@ -336,6 +359,8 @@ func (r *Runtime) ContainerStats(ctx context.Context, c *Container, cgroup strin
 
 // SignalContainer sends a signal to a container process.
 func (r *Runtime) SignalContainer(ctx context.Context, c *Container, sig syscall.Signal) error {
+	ctx, span := log.StartSpan(ctx)
+	defer span.End()
 	impl, err := r.RuntimeImpl(c)
 	if err != nil {
 		return err
@@ -346,6 +371,8 @@ func (r *Runtime) SignalContainer(ctx context.Context, c *Container, sig syscall
 
 // AttachContainer attaches IO to a running container.
 func (r *Runtime) AttachContainer(ctx context.Context, c *Container, inputStream io.Reader, outputStream, errorStream io.WriteCloser, tty bool, resize <-chan remotecommand.TerminalSize) error {
+	ctx, span := log.StartSpan(ctx)
+	defer span.End()
 	impl, err := r.RuntimeImpl(c)
 	if err != nil {
 		return err
@@ -356,6 +383,8 @@ func (r *Runtime) AttachContainer(ctx context.Context, c *Container, inputStream
 
 // PortForwardContainer forwards the specified port provides statistics of a container.
 func (r *Runtime) PortForwardContainer(ctx context.Context, c *Container, netNsPath string, port int32, stream io.ReadWriteCloser) error {
+	ctx, span := log.StartSpan(ctx)
+	defer span.End()
 	impl, err := r.RuntimeImpl(c)
 	if err != nil {
 		return err
@@ -366,6 +395,8 @@ func (r *Runtime) PortForwardContainer(ctx context.Context, c *Container, netNsP
 
 // ReopenContainerLog reopens the log file of a container.
 func (r *Runtime) ReopenContainerLog(ctx context.Context, c *Container) error {
+	ctx, span := log.StartSpan(ctx)
+	defer span.End()
 	impl, err := r.RuntimeImpl(c)
 	if err != nil {
 		return err
@@ -386,40 +417,22 @@ func (e *ExecSyncError) Error() string {
 	return fmt.Sprintf("command error: %+v, stdout: %s, stderr: %s, exit code %d", e.Err, e.Stdout.Bytes(), e.Stderr.Bytes(), e.ExitCode)
 }
 
-// BuildContainerdBinaryName() is responsible for ensuring the binary passed will
-// be properly converted to the containerd binary naming pattern.
-//
-// This method should never ever be called from anywhere else but the runtimeVM,
-// and the only reason its exported here is in order to get some test coverage.
-func BuildContainerdBinaryName(path string) string {
-	// containerd expects the runtime name to be in the following pattern:
-	//        ($dir.)?$prefix.$name.$version
-	//        -------- ------ ----- ---------
-	//              |     |     |      |
-	//              v     |     |      |
-	//      "/usr/local/bin"    |      |
-	//        (optional)  |     |      |
-	//                    v     |      |
-	//             "containerd.shim."  |
-	//                          |      |
-	//                          v      |
-	//                     "kata-qemu" |
-	//                                 v
-	//                                "v2"
-	const expectedPrefix = "containerd-shim-"
-	const expectedVersion = "-v2"
+// CheckpointContainer checkpoints a container.
+func (r *Runtime) CheckpointContainer(ctx context.Context, c *Container, specgen *rspec.Spec, leaveRunning bool) error {
+	impl, err := r.RuntimeImpl(c)
+	if err != nil {
+		return err
+	}
 
-	const binaryPrefix = "containerd.shim"
-	const binaryVersion = "v2"
+	return impl.CheckpointContainer(ctx, c, specgen, leaveRunning)
+}
 
-	runtimeDir := filepath.Dir(path)
-	// This is only safe to do because the runtime_path, for the VM runtime_type, is validated in the config,
-	// allowing us to take the liberty to simply go ahead and check, without having to ensure we're receiving
-	// the binary in the expected form.
-	//
-	// For clarity, it could be ensured twice, but we count on the developer to never ever call this function
-	// in a different context from the one used in the runtime_vm.go file.
-	runtimeName := strings.SplitAfter(strings.Split(filepath.Base(path), expectedVersion)[0], expectedPrefix)[1]
+// RestoreContainer restores a container.
+func (r *Runtime) RestoreContainer(ctx context.Context, c *Container, cgroupParent, mountLabel string) error {
+	impl, err := r.RuntimeImpl(c)
+	if err != nil {
+		return err
+	}
 
-	return filepath.Join(runtimeDir, fmt.Sprintf("%s.%s.%s", binaryPrefix, runtimeName, binaryVersion))
+	return impl.RestoreContainer(ctx, c, cgroupParent, mountLabel)
 }
