@@ -17,7 +17,6 @@ import (
 	metadata "github.com/checkpoint-restore/checkpointctl/lib"
 	"github.com/containernetworking/plugins/pkg/ns"
 	conmonconfig "github.com/containers/conmon/runner/config"
-	"github.com/containers/podman/v4/pkg/annotations"
 	"github.com/containers/podman/v4/pkg/checkpoint/crutils"
 	"github.com/containers/podman/v4/pkg/criu"
 	"github.com/containers/storage/pkg/pools"
@@ -30,7 +29,6 @@ import (
 	"github.com/fsnotify/fsnotify"
 	json "github.com/json-iterator/go"
 	rspec "github.com/opencontainers/runtime-spec/specs-go"
-	"github.com/opencontainers/runtime-tools/generate"
 	"github.com/sirupsen/logrus"
 	"golang.org/x/net/context"
 	"golang.org/x/sys/unix"
@@ -86,6 +84,9 @@ type exitCodeInfo struct {
 
 // CreateContainer creates a container.
 func (r *runtimeOCI) CreateContainer(ctx context.Context, c *Container, cgroupParent string, restore bool) (retErr error) {
+	ctx, span := log.StartSpan(ctx)
+	defer span.End()
+
 	if c.Spoofed() {
 		return nil
 	}
@@ -140,7 +141,7 @@ func (r *runtimeOCI) CreateContainer(ctx context.Context, c *Container, cgroupPa
 		args = append(args, "-i")
 	}
 	if restore {
-		logrus.Debugf("Restore is true %v", restore)
+		log.Debugf(ctx, "Restore is true %v", restore)
 		args = append(args, "--restore", c.CheckpointPath())
 		if c.Spec().Process.SelinuxLabel != "" {
 			args = append(
@@ -164,7 +165,7 @@ func (r *runtimeOCI) CreateContainer(ctx context.Context, c *Container, cgroupPa
 		}
 	}
 
-	logrus.WithFields(logrus.Fields{
+	log.WithFields(ctx, logrus.Fields{
 		"args": args,
 	}).Debugf("running conmon: %s", r.handler.MonitorPath)
 
@@ -189,7 +190,7 @@ func (r *runtimeOCI) CreateContainer(ctx context.Context, c *Container, cgroupPa
 	if restore {
 		// The CRIU binary is usually in /usr/sbin/criu
 		if v, found := os.LookupEnv("PATH"); found {
-			cmd.Env = append(cmd.Env, fmt.Sprintf("PATH=/usr/sbin/%s", v))
+			cmd.Env = append(cmd.Env, fmt.Sprintf("PATH=%s", v))
 		}
 	}
 
@@ -224,8 +225,10 @@ func (r *runtimeOCI) CreateContainer(ctx context.Context, c *Container, cgroupPa
 			}
 		}()
 		// Platform specific container setup
-		if err := r.createContainerPlatform(c, cgroupParent, cmd.Process.Pid); err != nil {
-			return err
+		if !c.Spoofed() {
+			if err := r.createContainerPlatform(c, cgroupParent, cmd.Process.Pid); err != nil {
+				return err
+			}
 		}
 
 		/* We set the cgroup, now the child can start creating children */
@@ -309,6 +312,8 @@ func (r *runtimeOCI) CreateContainer(ctx context.Context, c *Container, cgroupPa
 
 // StartContainer starts a container.
 func (r *runtimeOCI) StartContainer(ctx context.Context, c *Container) error {
+	_, span := log.StartSpan(ctx)
+	defer span.End()
 	c.opLock.Lock()
 	defer c.opLock.Unlock()
 
@@ -343,6 +348,8 @@ func prepareExec() (pidFileName string, parentPipe, childPipe *os.File, _ error)
 }
 
 func parseLog(ctx context.Context, l []byte) (stdout, stderr []byte) {
+	ctx, span := log.StartSpan(ctx)
+	defer span.End()
 	// Split the log on newlines, which is what separates entries.
 	lines := bytes.SplitAfter(l, []byte{'\n'})
 	for _, line := range lines {
@@ -388,6 +395,9 @@ func parseLog(ctx context.Context, l []byte) (stdout, stderr []byte) {
 
 // ExecContainer prepares a streaming endpoint to execute a command in the container.
 func (r *runtimeOCI) ExecContainer(ctx context.Context, c *Container, cmd []string, stdin io.Reader, stdout, stderr io.WriteCloser, tty bool, resize <-chan remotecommand.TerminalSize) error {
+	_, span := log.StartSpan(ctx)
+	defer span.End()
+
 	if c.Spoofed() {
 		return nil
 	}
@@ -468,6 +478,9 @@ func (r *runtimeOCI) ExecContainer(ctx context.Context, c *Container, cmd []stri
 
 // ExecSyncContainer execs a command in a container and returns it's stdout, stderr and return code.
 func (r *runtimeOCI) ExecSyncContainer(ctx context.Context, c *Container, command []string, timeout int64) (*types.ExecSyncResponse, error) {
+	ctx, span := log.StartSpan(ctx)
+	defer span.End()
+
 	if c.Spoofed() {
 		return nil, nil
 	}
@@ -725,6 +738,8 @@ func (r *runtimeOCI) ExecSyncContainer(ctx context.Context, c *Container, comman
 }
 
 func TruncateAndReadFile(ctx context.Context, path string, size int64) ([]byte, error) {
+	ctx, span := log.StartSpan(ctx)
+	defer span.End()
 	info, err := os.Stat(path)
 	if err != nil {
 		return nil, err
@@ -740,6 +755,9 @@ func TruncateAndReadFile(ctx context.Context, path string, size int64) ([]byte,
 
 // UpdateContainer updates container resources
 func (r *runtimeOCI) UpdateContainer(ctx context.Context, c *Container, res *rspec.LinuxResources) error {
+	_, span := log.StartSpan(ctx)
+	defer span.End()
+
 	c.opLock.Lock()
 	defer c.opLock.Unlock()
 
@@ -768,6 +786,9 @@ func (r *runtimeOCI) UpdateContainer(ctx context.Context, c *Container, res *rsp
 }
 
 func WaitContainerStop(ctx context.Context, c *Container, timeout time.Duration, ignoreKill bool) error {
+	ctx, span := log.StartSpan(ctx)
+	defer span.End()
+
 	done := make(chan struct{})
 	// we could potentially re-use "done" channel to exit the loop on timeout,
 	// but we use another channel "chControl" so that we never panic
@@ -847,6 +868,8 @@ func WaitContainerStop(ctx context.Context, c *Container, timeout time.Duration,
 
 // StopContainer stops a container. Timeout is given in seconds.
 func (r *runtimeOCI) StopContainer(ctx context.Context, c *Container, timeout int64) (retErr error) {
+	ctx, span := log.StartSpan(ctx)
+	defer span.End()
 	if c.SetAsStopping(timeout) {
 		return nil
 	}
@@ -908,6 +931,8 @@ func checkProcessGone(c *Container) {
 
 // DeleteContainer deletes a container.
 func (r *runtimeOCI) DeleteContainer(ctx context.Context, c *Container) error {
+	_, span := log.StartSpan(ctx)
+	defer span.End()
 	c.opLock.Lock()
 	defer c.opLock.Unlock()
 
@@ -943,6 +968,8 @@ func updateContainerStatusFromExitFile(c *Container) error {
 
 // UpdateContainerStatus refreshes the status of the container.
 func (r *runtimeOCI) UpdateContainerStatus(ctx context.Context, c *Container) error {
+	ctx, span := log.StartSpan(ctx)
+	defer span.End()
 	c.opLock.Lock()
 	defer c.opLock.Unlock()
 
@@ -1077,6 +1104,8 @@ func (r *runtimeOCI) UnpauseContainer(ctx context.Context, c *Container) error {
 
 // ContainerStats provides statistics of a container.
 func (r *runtimeOCI) ContainerStats(ctx context.Context, c *Container, cgroup string) (*types.ContainerStats, error) {
+	_, span := log.StartSpan(ctx)
+	defer span.End()
 	c.opLock.Lock()
 	defer c.opLock.Unlock()
 	return r.containerStats(c, cgroup)
@@ -1084,6 +1113,8 @@ func (r *runtimeOCI) ContainerStats(ctx context.Context, c *Container, cgroup st
 
 // SignalContainer sends a signal to a container process.
 func (r *runtimeOCI) SignalContainer(ctx context.Context, c *Container, sig syscall.Signal) error {
+	_, span := log.StartSpan(ctx)
+	defer span.End()
 	c.opLock.Lock()
 	defer c.opLock.Unlock()
 
@@ -1112,6 +1143,8 @@ func (r *runtimeOCI) signalContainer(c *Container, sig syscall.Signal, all bool)
 
 // AttachContainer attaches IO to a running container.
 func (r *runtimeOCI) AttachContainer(ctx context.Context, c *Container, inputStream io.Reader, outputStream, errorStream io.WriteCloser, tty bool, resize <-chan remotecommand.TerminalSize) error {
+	ctx, span := log.StartSpan(ctx)
+	defer span.End()
 	if c.Spoofed() {
 		return nil
 	}
@@ -1190,6 +1223,8 @@ func (r *runtimeOCI) AttachContainer(ctx context.Context, c *Container, inputStr
 
 // PortForwardContainer forwards the specified port into the provided container.
 func (r *runtimeOCI) PortForwardContainer(ctx context.Context, c *Container, netNsPath string, port int32, stream io.ReadWriteCloser) error {
+	ctx, span := log.StartSpan(ctx)
+	defer span.End()
 	log.Infof(ctx,
 		"Starting port forward for %s in network namespace %s", c.ID(), netNsPath,
 	)
@@ -1225,7 +1260,7 @@ func (r *runtimeOCI) PortForwardContainer(ctx context.Context, c *Container, net
 			), args...)
 		}
 
-		// Copy from the the namespace port connection to the client stream
+		// Copy from the namespace port connection to the client stream
 		go func() {
 			debug("copy data from container to client")
 			_, err := io.Copy(stream, conn)
@@ -1281,6 +1316,8 @@ func (r *runtimeOCI) PortForwardContainer(ctx context.Context, c *Container, net
 
 // ReopenContainerLog reopens the log file of a container.
 func (r *runtimeOCI) ReopenContainerLog(ctx context.Context, c *Container) error {
+	ctx, span := log.StartSpan(ctx)
+	defer span.End()
 	if c.Spoofed() {
 		return nil
 	}
@@ -1459,8 +1496,8 @@ func (r *runtimeOCI) CheckpointContainer(ctx context.Context, c *Container, spec
 	// imagePath is used by CRIU to store the actual checkpoint files
 	imagePath := c.CheckpointPath()
 
-	logrus.Debugf("Writing checkpoint to %s", imagePath)
-	logrus.Debugf("Writing checkpoint logs to %s", workPath)
+	log.Debugf(ctx, "Writing checkpoint to %s", imagePath)
+	log.Debugf(ctx, "Writing checkpoint logs to %s", workPath)
 	args := []string{}
 	args = append(
 		args,
@@ -1491,7 +1528,7 @@ func (r *runtimeOCI) CheckpointContainer(ctx context.Context, c *Container, spec
 }
 
 // RestoreContainer restores a container.
-func (r *runtimeOCI) RestoreContainer(ctx context.Context, c *Container, sbSpec *rspec.Spec, infraPid int, cgroupParent string) error {
+func (r *runtimeOCI) RestoreContainer(ctx context.Context, c *Container, cgroupParent, mountLabel string) error {
 	if err := r.checkpointRestoreSupported(); err != nil {
 		return err
 	}
@@ -1518,78 +1555,6 @@ func (r *runtimeOCI) RestoreContainer(ctx context.Context, c *Container, sbSpec
 		return fmt.Errorf("error removing container %s winsz file: %w", c.ID(), err)
 	}
 
-	// Figure out if this container will be restored in another sandbox
-	oldSbID := c.Sandbox()
-	if oldSbID == "" {
-		return fmt.Errorf("failed to detect sandbox of to be restored container %s", c.ID())
-	}
-	newSbID := sbSpec.Annotations[annotations.SandboxID]
-	if newSbID == "" {
-		return fmt.Errorf("failed to detect destination sandbox of to be restored container %s", c.ID())
-	}
-
-	// Get config.json to adapt for restore (mostly annotations for restore in another sandbox)
-	configFile := filepath.Join(c.BundlePath(), "config.json")
-	specgen, err := generate.NewFromFile(configFile)
-	if err != nil {
-		return err
-	}
-
-	if oldSbID != newSbID {
-		// The container will be restored in another (not the original) sandbox
-		// Adapt to namespaces of the new sandbox
-		for i, n := range specgen.Config.Linux.Namespaces {
-			if n.Path == "" {
-				// The namespace in the original container did not point to
-				// an existing interface. Leave it as it is.
-				continue
-			}
-			for _, on := range sbSpec.Linux.Namespaces {
-				if on.Type == n.Type {
-					var nsPath string
-					if n.Type == rspec.NetworkNamespace {
-						// Type for network namespaces is 'network'.
-						// The kernel link is 'net'.
-						nsPath = fmt.Sprintf("/proc/%d/ns/%s", infraPid, "net")
-					} else {
-						nsPath = fmt.Sprintf("/proc/%d/ns/%s", infraPid, n.Type)
-					}
-					specgen.Config.Linux.Namespaces[i].Path = nsPath
-					break
-				}
-			}
-		}
-
-		// Update Sandbox Name
-		specgen.AddAnnotation(annotations.SandboxName, sbSpec.Annotations[annotations.Name])
-		// Update Sandbox ID
-		specgen.AddAnnotation(annotations.SandboxID, newSbID)
-
-		// Update Name
-		ctrMetadata := types.ContainerMetadata{}
-		err = json.Unmarshal([]byte(sbSpec.Annotations[annotations.Metadata]), &ctrMetadata)
-		if err != nil {
-			return err
-		}
-		ctrName := ctrMetadata.Name
-
-		podMetadata := types.PodSandboxMetadata{}
-		err = json.Unmarshal([]byte(specgen.Config.Annotations[annotations.Metadata]), &podMetadata)
-		if err != nil {
-			return err
-		}
-		uid := podMetadata.Uid
-		mData := fmt.Sprintf("k8s_%s_%s_%s_%s0", ctrName, sbSpec.Annotations[annotations.KubeName], sbSpec.Annotations[annotations.Namespace], uid)
-		specgen.AddAnnotation(annotations.Name, mData)
-
-		c.SetSandbox(newSbID)
-
-		saveOptions := generate.ExportOptions{}
-		if err := specgen.SaveToFile(configFile, saveOptions); err != nil {
-			return err
-		}
-	}
-
 	c.state.InitPid = 0
 	c.state.InitStartTime = ""
 
@@ -1602,7 +1567,7 @@ func (r *runtimeOCI) RestoreContainer(ctx context.Context, c *Container, sbSpec
 	if err := crutils.CRCreateFileWithLabel(
 		c.BundlePath(),
 		metadata.RestoreLogFile,
-		specgen.Config.Linux.MountLabel,
+		mountLabel,
 	); err != nil {
 		return err
 	}
