@@ -13,6 +13,7 @@ import (
 	"github.com/containers/common/pkg/resize"
 	conmonClient "github.com/containers/conmon-rs/pkg/client"
 	conmonconfig "github.com/containers/conmon/runner/config"
+	"github.com/cri-o/cri-o/internal/log"
 	"github.com/cri-o/cri-o/pkg/config"
 	rspec "github.com/opencontainers/runtime-spec/specs-go"
 	"github.com/sirupsen/logrus"
@@ -58,10 +59,15 @@ func newRuntimePod(r *Runtime, handler *config.RuntimeHandler, c *Container) (Ru
 	client, err := conmonClient.New(&conmonClient.ConmonServerConfig{
 		ConmonServerPath: handler.MonitorPath,
 		LogLevel:         conmonClient.FromLogrusLevel(logrus.GetLevel()),
+		LogDriver:        conmonClient.LogDriverSystemd,
 		Runtime:          handler.RuntimePath,
 		ServerRunDir:     c.dir,
 		RuntimeRoot:      runRoot,
 		CgroupManager:    cgroupManager,
+		Tracing: &conmonClient.Tracing{
+			Enabled:  r.config.EnableTracing,
+			Endpoint: fmt.Sprintf("http://%s", r.config.TracingEndpoint),
+		},
 	})
 	if err != nil {
 		return nil, err
@@ -81,16 +87,23 @@ func newRuntimePod(r *Runtime, handler *config.RuntimeHandler, c *Container) (Ru
 }
 
 func (r *runtimePod) CreateContainer(ctx context.Context, c *Container, cgroupParent string, restore bool) error {
-	// If this container is the infra container, all that needs to be done is move conmonrs to the pod cgroup
-	if c.IsInfra() {
-		v, err := r.client.Version(ctx)
+	// If this container is the infra container or spoofed,
+	// then it is the pod container and we move conmonrs
+	// to the right configured cgroup.
+	if c.IsInfra() || c.Spoofed() {
+		v, err := r.client.Version(ctx, &conmonClient.VersionConfig{Verbose: false})
 		if err != nil {
 			return fmt.Errorf("failed to get version of client before moving server to cgroup: %w", err)
 		}
 
-		logrus.Debugf(
-			"Using conmonrs version: %s, tag: %s, commit: %s, build: %s, rustc: %s",
-			v.Version, v.Tag, v.Commit, v.BuildDate, v.RustVersion,
+		if v.Tag == "" {
+			v.Tag = "none"
+		}
+
+		log.Debugf(
+			ctx,
+			"Using conmonrs version: %s, tag: %s, commit: %s, build: %s, target: %s, %s, %s",
+			v.Version, v.Tag, v.Commit, v.BuildDate, v.Target, v.RustVersion, v.CargoVersion,
 		)
 
 		// Platform specific container setup
@@ -109,9 +122,10 @@ func (r *runtimePod) CreateContainer(ctx context.Context, c *Container, cgroupPa
 		ID:           c.ID(),
 		BundlePath:   c.bundlePath,
 		Terminal:     c.terminal,
+		Stdin:        c.stdin,
 		ExitPaths:    []string{filepath.Join(r.oci.config.ContainerExitsDir, c.ID()), c.exitFilePath()},
 		OOMExitPaths: []string{filepath.Join(c.bundlePath, "oom")}, // Keep in sync with location in oci.UpdateContainerStatus()
-		LogDrivers: []conmonClient.LogDriver{
+		LogDrivers: []conmonClient.ContainerLogDriver{
 			{
 				Type:    conmonClient.LogDriverTypeContainerRuntimeInterface,
 				Path:    c.logPath,
@@ -147,11 +161,10 @@ func (r *runtimePod) CheckpointContainer(
 func (r *runtimePod) RestoreContainer(
 	ctx context.Context,
 	c *Container,
-	sbSpec *rspec.Spec,
-	infraPid int,
 	cgroupParent string,
+	mountLabel string,
 ) error {
-	return r.oci.RestoreContainer(ctx, c, sbSpec, infraPid, cgroupParent)
+	return r.oci.RestoreContainer(ctx, c, cgroupParent, mountLabel)
 }
 
 func (r *runtimePod) ExecContainer(ctx context.Context, c *Container, cmd []string, stdin io.Reader, stdout, stderr io.WriteCloser, tty bool, resizeChan <-chan remotecommand.TerminalSize) error {
@@ -249,7 +262,7 @@ func (r *runtimePod) AttachContainer(ctx context.Context, c *Container, inputStr
 	)
 
 	if inputStream != nil {
-		stdin = &conmonClient.In{Reader: inputStream}
+		stdin = &conmonClient.In{ReadCloser: io.NopCloser(inputStream)}
 	}
 	if outputStream != nil {
 		stdout = &conmonClient.Out{WriteCloser: outputStream}
@@ -262,7 +275,8 @@ func (r *runtimePod) AttachContainer(ctx context.Context, c *Container, inputStr
 		ID:                c.ID(),
 		SocketPath:        attachSocketPath,
 		Tty:               tty,
-		StopAfterStdinEOF: c.stdin && !c.StdinOnce() && !tty,
+		StopAfterStdinEOF: c.stdin && c.StdinOnce() && !tty,
+		ContainerStdin:    c.stdin,
 		Resize:            libpodResize,
 		Streams: conmonClient.AttachStreams{
 			Stdin:  stdin,
