@@ -1,10 +1,12 @@
 package resourcestore
 
 import (
+	"context"
 	"fmt"
 	"sync"
 	"time"
 
+	"github.com/cri-o/cri-o/internal/log"
 	"github.com/sirupsen/logrus"
 )
 
@@ -177,6 +179,15 @@ func (rc *ResourceStore) Put(name string, resource IdentifiableCreatable, cleane
 	return nil
 }
 
+// Delete deletes the specified resource from the store.
+// Any resource that has a stage set, but was never Put should have Delete called, or else it will leak.
+func (rc *ResourceStore) Delete(name string) {
+	rc.mutex.Lock()
+	defer rc.mutex.Unlock()
+
+	delete(rc.resources, name)
+}
+
 // WatcherForResource looks up a Resource by name, and gives it a watcher.
 // If no entry exists for that resource, a placeholder is created and a watcher is given to that
 // placeholder resource.
@@ -200,12 +211,12 @@ func (rc *ResourceStore) WatcherForResource(name string) (watcher chan struct{},
 	return watcher, r.stage
 }
 
-func (rc *ResourceStore) SetStageForResource(name, stage string) {
+func (rc *ResourceStore) SetStageForResource(ctx context.Context, name, stage string) {
 	rc.mutex.Lock()
 	defer rc.mutex.Unlock()
 	r, ok := rc.resources[name]
 	if !ok {
-		logrus.Tracef("Initializing stage for resource %s to %s", name, stage)
+		log.Debugf(ctx, "Initializing stage for resource %s to %s", name, stage)
 		rc.resources[name] = &Resource{
 			watchers: []chan struct{}{},
 			name:     name,
@@ -213,6 +224,6 @@ func (rc *ResourceStore) SetStageForResource(name, stage string) {
 		}
 		return
 	}
-	logrus.Tracef("Setting stage for resource %s from %s to %s", name, r.stage, stage)
+	log.Debugf(ctx, "Setting stage for resource %s from %s to %s", name, r.stage, stage)
 	r.stage = stage
 }
