@@ -2,8 +2,8 @@ package config
 
 import (
 	"bytes"
+	"errors"
 	"fmt"
-	"io/ioutil"
 	"net"
 	"os"
 	"os/exec"
@@ -12,11 +12,12 @@ import (
 	"strings"
 
 	"github.com/BurntSushi/toml"
+	"github.com/container-orchestrated-devices/container-device-interface/pkg/cdi"
+	"github.com/containers/common/pkg/hooks"
 	conmonconfig "github.com/containers/conmon/runner/config"
 	"github.com/containers/image/v5/pkg/sysregistriesv2"
 	"github.com/containers/image/v5/types"
-	"github.com/containers/podman/v3/pkg/hooks"
-	"github.com/containers/podman/v3/pkg/rootless"
+	"github.com/containers/podman/v4/pkg/rootless"
 	"github.com/containers/storage"
 	"github.com/cri-o/cri-o/internal/config/apparmor"
 	"github.com/cri-o/cri-o/internal/config/blockio"
@@ -26,18 +27,18 @@ import (
 	"github.com/cri-o/cri-o/internal/config/conmonmgr"
 	"github.com/cri-o/cri-o/internal/config/device"
 	"github.com/cri-o/cri-o/internal/config/node"
+	"github.com/cri-o/cri-o/internal/config/nri"
 	"github.com/cri-o/cri-o/internal/config/nsmgr"
 	"github.com/cri-o/cri-o/internal/config/rdt"
 	"github.com/cri-o/cri-o/internal/config/seccomp"
 	"github.com/cri-o/cri-o/internal/config/ulimits"
 	"github.com/cri-o/cri-o/pkg/annotations"
-	"github.com/cri-o/cri-o/server/metrics/collectors"
+	"github.com/cri-o/cri-o/server/otel-collector/collectors"
 	"github.com/cri-o/cri-o/server/useragent"
 	"github.com/cri-o/cri-o/utils"
 	"github.com/cri-o/cri-o/utils/cmdrunner"
 	"github.com/cri-o/ocicni/pkg/ocicni"
 	selinux "github.com/opencontainers/selinux/go-selinux"
-	"github.com/pkg/errors"
 	"github.com/sirupsen/logrus"
 
 	"k8s.io/kubernetes/pkg/kubelet/cm/cpuset"
@@ -48,18 +49,23 @@ const (
 	defaultRuntime             = "runc"
 	DefaultRuntimeType         = "oci"
 	DefaultRuntimeRoot         = "/run/runc"
-	defaultGRPCMaxMsgSize      = 16 * 1024 * 1024
+	defaultGRPCMaxMsgSize      = 80 * 1024 * 1024
 	OCIBufSize                 = 8192
 	RuntimeTypeVM              = "vm"
+	RuntimeTypePod             = "pod"
 	defaultCtrStopTimeout      = 30 // seconds
 	defaultNamespacesDir       = "/var/run"
 	RuntimeTypeVMBinaryPattern = "containerd-shim-([a-zA-Z0-9\\-\\+])+-v2"
 	tasksetBinary              = "taskset"
+	defaultMonitorCgroup       = "system.slice"
+	MonitorExecCgroupDefault   = ""
+	MonitorExecCgroupContainer = "container"
 )
 
 // Config represents the entire set of configuration values that can be set for
 // the server. This is intended to be loaded from a toml-encoded config file.
 type Config struct {
+	Comment          string
 	singleConfigPath string // Path to the single config file
 	dropInConfigDir  string // Path to the drop-in config files
 
@@ -69,6 +75,9 @@ type Config struct {
 	ImageConfig
 	NetworkConfig
 	MetricsConfig
+	TracingConfig
+	StatsConfig
+	NRI           *nri.Config
 	SystemContext *types.SystemContext
 }
 
@@ -78,16 +87,6 @@ type Iface interface {
 	GetData() *Config
 }
 
-// GetStore returns the container storage for a given configuration
-func (c *RootConfig) GetStore() (storage.Store, error) {
-	return storage.GetStore(storage.StoreOptions{
-		RunRoot:            c.RunRoot,
-		GraphRoot:          c.Root,
-		GraphDriverName:    c.Storage,
-		GraphDriverOptions: c.StorageOptions,
-	})
-}
-
 // GetData returns the Config of a Iface
 func (c *Config) GetData() *Config {
 	return c
@@ -104,13 +103,13 @@ const (
 	// ImageVolumesBind option is for using bind mounted volumes
 	ImageVolumesBind ImageVolumesType = "bind"
 	// DefaultPauseImage is default pause image
-	DefaultPauseImage string = "k8s.gcr.io/pause:3.5"
+	DefaultPauseImage string = "registry.k8s.io/pause:3.6"
 )
 
 const (
 	// DefaultPidsLimit is the default value for maximum number of processes
 	// allowed inside a container
-	DefaultPidsLimit = 1024
+	DefaultPidsLimit = 0
 
 	// DefaultLogSizeMax is the default value for the maximum log size
 	// allowed for a container. Negative values mean that no limit is imposed.
@@ -170,6 +169,16 @@ type RootConfig struct {
 	InternalWipe bool `toml:"internal_wipe"`
 }
 
+// GetStore returns the container storage for a given configuration
+func (c *RootConfig) GetStore() (storage.Store, error) {
+	return storage.GetStore(storage.StoreOptions{
+		RunRoot:            c.RunRoot,
+		GraphRoot:          c.Root,
+		GraphDriverName:    c.Storage,
+		GraphDriverOptions: c.StorageOptions,
+	})
+}
+
 // RuntimeHandler represents each item of the "crio.runtime.runtimes" TOML
 // config table.
 type RuntimeHandler struct {
@@ -188,10 +197,21 @@ type RuntimeHandler struct {
 	// "io.kubernetes.cri-o.ShmSize" for configuring the size of /dev/shm.
 	// "io.kubernetes.cri-o.UnifiedCgroup.$CTR_NAME" for configuring the cgroup v2 unified block for a container.
 	// "io.containers.trace-syscall" for tracing syscalls via the OCI seccomp BPF hook.
+	// "io.kubernetes.cri-o.LinkLogs" for linking logs into the pod.
 	AllowedAnnotations []string `toml:"allowed_annotations,omitempty"`
 
 	// DisallowedAnnotations is the slice of experimental annotations that are not allowed for this handler.
 	DisallowedAnnotations []string
+
+	// Fields prefixed by Monitor hold the configuration for the monitor for this runtime. At present, the following monitors are supported:
+	// oci supports conmon
+	// vm does not support any runtime monitor
+	MonitorPath   string   `toml:"monitor_path,omitempty"`
+	MonitorCgroup string   `toml:"monitor_cgroup,omitempty"`
+	MonitorEnv    []string `toml:"monitor_env,omitempty"`
+
+	// MonitorExecCgroup indicates whether to move exec probes to the container's cgroup.
+	MonitorExecCgroup string `toml:"monitor_exec_cgroup,omitempty"`
 }
 
 // Multiple runtime Handlers in a map
@@ -223,10 +243,11 @@ type RuntimeConfig struct {
 	ReadOnly bool `toml:"read_only"`
 
 	// ConmonEnv is the environment variable list for conmon process.
+	// This option is currently deprecated, and will be replaced with RuntimeHandler.MonitorEnv.
 	ConmonEnv []string `toml:"conmon_env"`
 
 	// HooksDir holds paths to the directories containing hooks
-	// configuration files.  When the same filename is present in in
+	// configuration files.  When the same filename is present in
 	// multiple directories, the file in the directory listed last in
 	// this slice takes precedence.
 	HooksDir []string `toml:"hooks_dir"`
@@ -234,6 +255,10 @@ type RuntimeConfig struct {
 	// Capabilities to add to all containers.
 	DefaultCapabilities capabilities.Capabilities `toml:"default_capabilities"`
 
+	// AddInheritableCapabilities can be set to add inheritable capabilities. They were pre-1.23 by default, and were dropped in 1.24.
+	// This can cause a regression with non-root users not getting capabilities as they previously did.
+	AddInheritableCapabilities bool `toml:"add_inheritable_capabilities"`
+
 	// Additional environment variables to set for all the
 	// containers. These are overridden if set in the
 	// container image spec or in the container runtime configuration.
@@ -245,9 +270,15 @@ type RuntimeConfig struct {
 	// DefaultUlimits specifies the default ulimits to apply to containers
 	DefaultUlimits []string `toml:"default_ulimits"`
 
+	// Devices that are allowed to be configured.
+	AllowedDevices []string `toml:"allowed_devices"`
+
 	// Devices to add to containers
 	AdditionalDevices []string `toml:"additional_devices"`
 
+	// CDISpecDirs specifies the directories CRI-O/CDI will scan for CDI Spec files.
+	CDISpecDirs []string `toml:"cdi_spec_dirs"`
+
 	// DeviceOwnershipFromSecurityContext changes the default behavior of setting container devices uid/gid
 	// from CRI's SecurityContext (RunAsUser/RunAsGroup) instead of taking host's uid/gid. Defaults to false.
 	DeviceOwnershipFromSecurityContext bool `toml:"device_ownership_from_security_context"`
@@ -260,9 +291,11 @@ type RuntimeConfig struct {
 	DecryptionKeysPath string `toml:"decryption_keys_path"`
 
 	// Conmon is the path to conmon binary, used for managing the runtime.
+	// This option is currently deprecated, and will be replaced with RuntimeHandler.MonitorConfig.Path.
 	Conmon string `toml:"conmon"`
 
 	// ConmonCgroup is the cgroup setting used for conmon.
+	// This option is currently deprecated, and will be replaced with RuntimeHandler.MonitorConfig.Cgroup.
 	ConmonCgroup string `toml:"conmon_cgroup"`
 
 	// SeccompProfile is the seccomp.json profile path which is used as the
@@ -307,11 +340,21 @@ type RuntimeConfig struct {
 	// ranges are separated by comma.
 	UIDMappings string `toml:"uid_mappings"`
 
+	// MinimumMappableUID specifies the minimum UID value which can be
+	// specified in a uid_mappings value, whether configured here or sent
+	// to us via CRI, for a pod that isn't to be run as UID 0.
+	MinimumMappableUID int64 `toml:"minimum_mappable_uid"`
+
 	// GIDMappings specifies the GID mappings to have in the user namespace.
 	// A range is specified in the form containerUID:HostUID:Size.  Multiple
 	// ranges are separated by comma.
 	GIDMappings string `toml:"gid_mappings"`
 
+	// MinimumMappableGID specifies the minimum GID value which can be
+	// specified in a gid_mappings value, whether configured here or sent
+	// to us via CRI, for a pod that isn't to be run as UID 0.
+	MinimumMappableGID int64 `toml:"minimum_mappable_gid"`
+
 	// LogLevel determines the verbosity of the logs based on the level it is set to.
 	// Options are fatal, panic, error (default), warn, info, debug, and trace.
 	LogLevel string `toml:"log_level"`
@@ -327,6 +370,10 @@ type RuntimeConfig struct {
 	// to manage namespace lifecycle
 	PinnsPath string `toml:"pinns_path"`
 
+	// CriuPath is the path to find the criu binary, which is needed
+	// to checkpoint and restore containers
+	EnableCriuSupport bool `toml:"enable_criu_support"`
+
 	// Runtimes defines a list of OCI compatible runtimes. The runtime to
 	// use is picked based on the runtime_handler provided by the CRI. If
 	// no runtime_handler is provided, the runtime will be picked based on
@@ -361,6 +408,9 @@ type RuntimeConfig struct {
 	// will cause a container creation to fail (as opposed to the current behavior of creating a directory).
 	AbsentMountSourcesToReject []string `toml:"absent_mount_sources_to_reject"`
 
+	// EnablePodEvents specifies if the container pod-level events should be generated to optimize the PLEG at Kubelet.
+	EnablePodEvents bool `toml:"enable_pod_events"`
+
 	// seccompConfig is the internal seccomp configuration
 	seccompConfig *seccomp.Config
 
@@ -501,6 +551,26 @@ type MetricsConfig struct {
 	MetricsKey string `toml:"metrics_key"`
 }
 
+// TracingConfig specifies all necessary configuration for opentelemetry trace exports
+type TracingConfig struct {
+	// EnableTracing can be used to globally enable or disable tracing support
+	EnableTracing bool `toml:"enable_tracing"`
+
+	// TracingEndpoint is the address on which the grpc tracing collector server will listen.
+	TracingEndpoint string `toml:"tracing_endpoint"`
+
+	// TracingSamplingRatePerMillion is the number of samples to collect per million spans. Set to 1000000 to always sample.
+	// Defaults to 0.
+	TracingSamplingRatePerMillion int `toml:"tracing_sampling_rate_per_million"`
+}
+
+// StatsConfig specifies all necessary configuration for reporting container and pod stats
+type StatsConfig struct {
+	// StatsCollectionPeriod is the number of seconds between collecting pod and container stats.
+	// If set to 0, the stats are collected on-demand instead.
+	StatsCollectionPeriod int `toml:"stats_collection_period"`
+}
+
 // tomlConfig is another way of looking at a Config, which is
 // TOML-friendly (it has all of the explicit tables). It's just used for
 // conversions.
@@ -512,6 +582,9 @@ type tomlConfig struct {
 		Image   struct{ ImageConfig }   `toml:"image"`
 		Network struct{ NetworkConfig } `toml:"network"`
 		Metrics struct{ MetricsConfig } `toml:"metrics"`
+		Tracing struct{ TracingConfig } `toml:"tracing"`
+		Stats   struct{ StatsConfig }   `toml:"stats"`
+		NRI     struct{ *nri.Config }   `toml:"nri"`
 	} `toml:"crio"`
 }
 
@@ -521,12 +594,16 @@ func (t *tomlConfig) SetSystemContext(c *Config) {
 }
 
 func (t *tomlConfig) toConfig(c *Config) {
+	c.Comment = "# "
 	c.RootConfig = t.Crio.RootConfig
 	c.APIConfig = t.Crio.API.APIConfig
 	c.RuntimeConfig = t.Crio.Runtime.RuntimeConfig
 	c.ImageConfig = t.Crio.Image.ImageConfig
 	c.NetworkConfig = t.Crio.Network.NetworkConfig
 	c.MetricsConfig = t.Crio.Metrics.MetricsConfig
+	c.TracingConfig = t.Crio.Tracing.TracingConfig
+	c.StatsConfig = t.Crio.Stats.StatsConfig
+	c.NRI = t.Crio.NRI.Config
 	t.SetSystemContext(c)
 }
 
@@ -537,6 +614,9 @@ func (t *tomlConfig) fromConfig(c *Config) {
 	t.Crio.Image.ImageConfig = c.ImageConfig
 	t.Crio.Network.NetworkConfig = c.NetworkConfig
 	t.Crio.Metrics.MetricsConfig = c.MetricsConfig
+	t.Crio.Tracing.TracingConfig = c.TracingConfig
+	t.Crio.Stats.StatsConfig = c.StatsConfig
+	t.Crio.NRI.Config = c.NRI
 }
 
 // UpdateFromFile populates the Config from the TOML-encoded file at the given
@@ -561,8 +641,12 @@ func (c *Config) UpdateFromDropInFile(path string) error {
 	// keeps the storage options from storage.conf and merge it to crio config
 	var storageOpts []string
 	storageOpts = append(storageOpts, c.StorageOptions...)
+	// storage configurations from storage.conf, if crio config has no values for these, they will be merged to crio config
+	graphRoot := c.Root
+	runRoot := c.RunRoot
+	storageDriver := c.Storage
 
-	data, err := ioutil.ReadFile(path)
+	data, err := os.ReadFile(path)
 	if err != nil {
 		return err
 	}
@@ -570,20 +654,24 @@ func (c *Config) UpdateFromDropInFile(path string) error {
 	t := new(tomlConfig)
 	t.fromConfig(c)
 
-	metadata, err := toml.Decode(string(data), t)
+	_, err = toml.Decode(string(data), t)
 	if err != nil {
-		return fmt.Errorf("unable to decode configuration %v: %v", path, err)
-	}
-
-	runtimesKey := []string{"crio", "runtime", "default_runtime"}
-	if metadata.IsDefined(runtimesKey...) &&
-		t.Crio.Runtime.RuntimeConfig.DefaultRuntime != defaultRuntime {
-		delete(c.Runtimes, defaultRuntime)
+		return fmt.Errorf("unable to decode configuration %v: %w", path, err)
 	}
 
 	storageOpts = append(storageOpts, t.Crio.RootConfig.StorageOptions...)
 	storageOpts = removeDupStorageOpts(storageOpts)
 	t.Crio.RootConfig.StorageOptions = storageOpts
+	// inherits storage configurations from storage.conf
+	if t.Crio.Root == "" {
+		t.Crio.Root = graphRoot
+	}
+	if t.Crio.RunRoot == "" {
+		t.Crio.RunRoot = runRoot
+	}
+	if t.Crio.Storage == "" {
+		t.Crio.Storage = storageDriver
+	}
 
 	// Registries are deprecated in cri-o.conf and turned into a NOP.
 	// Users should use registries.conf instead, so let's log it.
@@ -645,7 +733,7 @@ func (c *Config) ToFile(path string) error {
 		return err
 	}
 
-	return ioutil.WriteFile(path, b, 0o644)
+	return os.WriteFile(path, b, 0o644)
 }
 
 // ToBytes encodes the config into a byte slice. It errors if the encoding
@@ -671,20 +759,25 @@ func DefaultConfig() (*Config, error) {
 		return nil, err
 	}
 	cgroupManager := cgmgr.New()
+	seccompConfig := seccomp.New()
+	ua, err := useragent.Get()
+	if err != nil {
+		return nil, fmt.Errorf("get user agent: %w", err)
+	}
 	return &Config{
+		Comment: "# ",
 		SystemContext: &types.SystemContext{
-			DockerRegistryUserAgent: useragent.Get(),
+			DockerRegistryUserAgent: ua,
 		},
 		RootConfig: RootConfig{
-			Root:               storeOpts.GraphRoot,
-			RunRoot:            storeOpts.RunRoot,
-			Storage:            storeOpts.GraphDriverName,
-			StorageOptions:     storeOpts.GraphDriverOptions,
-			LogDir:             "/var/log/crio/pods",
-			VersionFile:        CrioVersionPathTmp,
-			VersionFilePersist: CrioVersionPathPersist,
-			CleanShutdownFile:  CrioCleanShutdownFile,
-			InternalWipe:       true,
+			Root:              storeOpts.GraphRoot,
+			RunRoot:           storeOpts.RunRoot,
+			Storage:           storeOpts.GraphDriverName,
+			StorageOptions:    storeOpts.GraphDriverOptions,
+			LogDir:            "/var/log/crio/pods",
+			VersionFile:       CrioVersionPathTmp,
+			CleanShutdownFile: CrioCleanShutdownFile,
+			InternalWipe:      true,
 		},
 		APIConfig: APIConfig{
 			Listen:             CrioSocketPath,
@@ -694,45 +787,40 @@ func DefaultConfig() (*Config, error) {
 			GRPCMaxRecvMsgSize: defaultGRPCMaxMsgSize,
 		},
 		RuntimeConfig: RuntimeConfig{
+			AllowedDevices:     []string{"/dev/fuse"},
 			DecryptionKeysPath: "/etc/crio/keys/",
 			DefaultRuntime:     defaultRuntime,
 			Runtimes: Runtimes{
-				defaultRuntime: {
-					RuntimeType: DefaultRuntimeType,
-					RuntimeRoot: DefaultRuntimeRoot,
-					AllowedAnnotations: []string{
-						annotations.OCISeccompBPFHookAnnotation,
-					},
-				},
+				defaultRuntime: defaultRuntimeHandler(),
 			},
-			ConmonEnv: []string{
-				"PATH=/usr/local/sbin:/usr/local/bin:/usr/sbin:/usr/bin:/sbin:/bin",
-			},
-			ConmonCgroup:             "system.slice",
-			SELinux:                  selinuxEnabled(),
-			ApparmorProfile:          apparmor.DefaultProfile,
-			BlockIOConfigFile:        DefaultBlockIOConfigFile,
-			IrqBalanceConfigFile:     DefaultIrqBalanceConfigFile,
-			RdtConfigFile:            rdt.DefaultRdtConfigFile,
-			CgroupManagerName:        cgroupManager.Name(),
-			PidsLimit:                DefaultPidsLimit,
-			ContainerExitsDir:        containerExitsDir,
-			ContainerAttachSocketDir: conmonconfig.ContainerAttachSocketDir,
-			LogSizeMax:               DefaultLogSizeMax,
-			CtrStopTimeout:           defaultCtrStopTimeout,
-			DefaultCapabilities:      capabilities.Default(),
-			LogLevel:                 "info",
-			HooksDir:                 []string{hooks.DefaultDir},
-			NamespacesDir:            defaultNamespacesDir,
-			DropInfraCtr:             true,
-			seccompConfig:            seccomp.New(),
-			apparmorConfig:           apparmor.New(),
-			blockioConfig:            blockio.New(),
-			rdtConfig:                rdt.New(),
-			ulimitsConfig:            ulimits.New(),
-			cgroupManager:            cgroupManager,
-			deviceConfig:             device.New(),
-			namespaceManager:         nsmgr.New(defaultNamespacesDir, ""),
+			SELinux:                    selinuxEnabled(),
+			ApparmorProfile:            apparmor.DefaultProfile,
+			BlockIOConfigFile:          DefaultBlockIOConfigFile,
+			IrqBalanceConfigFile:       DefaultIrqBalanceConfigFile,
+			RdtConfigFile:              rdt.DefaultRdtConfigFile,
+			CgroupManagerName:          cgroupManager.Name(),
+			PidsLimit:                  DefaultPidsLimit,
+			ContainerExitsDir:          containerExitsDir,
+			ContainerAttachSocketDir:   conmonconfig.ContainerAttachSocketDir,
+			MinimumMappableUID:         -1,
+			MinimumMappableGID:         -1,
+			LogSizeMax:                 DefaultLogSizeMax,
+			CtrStopTimeout:             defaultCtrStopTimeout,
+			DefaultCapabilities:        capabilities.Default(),
+			LogLevel:                   "info",
+			HooksDir:                   []string{hooks.DefaultDir},
+			CDISpecDirs:                cdi.DefaultSpecDirs,
+			NamespacesDir:              defaultNamespacesDir,
+			DropInfraCtr:               true,
+			SeccompUseDefaultWhenEmpty: seccompConfig.UseDefaultWhenEmpty(),
+			seccompConfig:              seccomp.New(),
+			apparmorConfig:             apparmor.New(),
+			blockioConfig:              blockio.New(),
+			cgroupManager:              cgroupManager,
+			deviceConfig:               device.New(),
+			namespaceManager:           nsmgr.New(defaultNamespacesDir, ""),
+			rdtConfig:                  rdt.New(),
+			ulimitsConfig:              ulimits.New(),
 		},
 		ImageConfig: ImageConfig{
 			DefaultTransport: "docker://",
@@ -748,6 +836,12 @@ func DefaultConfig() (*Config, error) {
 			MetricsPort:       9090,
 			MetricsCollectors: collectors.All(),
 		},
+		TracingConfig: TracingConfig{
+			TracingEndpoint:               "0.0.0.0:4317",
+			TracingSamplingRatePerMillion: 0,
+			EnableTracing:                 false,
+		},
+		NRI: nri.New(),
 	}, nil
 }
 
@@ -771,25 +865,33 @@ func (c *Config) Validate(onExecution bool) error {
 	}
 
 	if err := c.RootConfig.Validate(onExecution); err != nil {
-		return errors.Wrap(err, "validating root config")
+		return fmt.Errorf("validating root config: %w", err)
 	}
 
 	if err := c.RuntimeConfig.Validate(c.SystemContext, onExecution); err != nil {
-		return errors.Wrap(err, "validating runtime config")
+		return fmt.Errorf("validating runtime config: %w", err)
 	}
 
+	c.RuntimeConfig.seccompConfig.SetNotifierPath(
+		filepath.Join(filepath.Dir(c.Listen), "seccomp"),
+	)
+
 	if err := c.NetworkConfig.Validate(onExecution); err != nil {
-		return errors.Wrap(err, "validating network config")
+		return fmt.Errorf("validating network config: %w", err)
 	}
 
 	if err := c.APIConfig.Validate(onExecution); err != nil {
-		return errors.Wrap(err, "validating api config")
+		return fmt.Errorf("validating api config: %w", err)
 	}
 
 	if !c.SELinux {
 		selinux.SetDisabled()
 	}
 
+	if err := c.NRI.Validate(onExecution); err != nil {
+		return fmt.Errorf("validating NRI config: %w", err)
+	}
+
 	return nil
 }
 
@@ -816,17 +918,17 @@ func (c *APIConfig) Validate(onExecution bool) error {
 // removes unused socket connections if available.
 func RemoveUnusedSocket(path string) error {
 	if err := os.MkdirAll(filepath.Dir(path), 0o755); err != nil {
-		return errors.Wrap(err, "creating socket directories")
+		return fmt.Errorf("creating socket directories: %w", err)
 	}
 
 	// Remove the socket if it already exists
 	if _, err := os.Stat(path); err == nil {
 		if _, err := net.DialTimeout("unix", path, 0); err == nil {
-			return errors.Errorf("already existing connection on %s", path)
+			return fmt.Errorf("already existing connection on %s", path)
 		}
 
 		if err := os.Remove(path); err != nil {
-			return errors.Wrapf(err, "removing %s", path)
+			return fmt.Errorf("removing %s: %w", path, err)
 		}
 	}
 
@@ -843,11 +945,11 @@ func (c *RootConfig) Validate(onExecution bool) error {
 			return errors.New("log_dir is not an absolute path")
 		}
 		if err := os.MkdirAll(c.LogDir, 0o700); err != nil {
-			return errors.Wrap(err, "invalid log_dir")
+			return fmt.Errorf("invalid log_dir: %w", err)
 		}
 		store, err := c.GetStore()
 		if err != nil {
-			return errors.Wrapf(err, "failed to get store to set defaults")
+			return fmt.Errorf("failed to get store to set defaults: %w", err)
 		}
 		// This step merges the /etc/container/storage.conf with the
 		// storage configuration in crio.conf
@@ -879,30 +981,8 @@ func (c *RuntimeConfig) Validate(systemContext *types.SystemContext, onExecution
 		return err
 	}
 
-	// check we do have at least a runtime
-	if _, ok := c.Runtimes[c.DefaultRuntime]; !ok {
-		// Set the default runtime to "runc" if default_runtime is not set
-		if c.DefaultRuntime == "" {
-			logrus.Debugf("Defaulting to %q as the runtime since default_runtime is not set", defaultRuntime)
-			// The default config sets runc and its path in the runtimes map, so check for that
-			// first. If it does not exist then we add runc + its path to the runtimes map.
-			if _, ok := c.Runtimes[defaultRuntime]; !ok {
-				c.Runtimes[defaultRuntime] = &RuntimeHandler{
-					RuntimePath:       "",
-					RuntimeType:       DefaultRuntimeType,
-					RuntimeRoot:       DefaultRuntimeRoot,
-					RuntimeConfigPath: "",
-				}
-			}
-			// Set the DefaultRuntime to runc so we don't fail further along in the code
-			c.DefaultRuntime = defaultRuntime
-		} else {
-			return fmt.Errorf("default_runtime set to %q, but no runtime entry was found for it", c.DefaultRuntime)
-		}
-	}
-
-	if !(c.ConmonCgroup == "pod" || strings.HasSuffix(c.ConmonCgroup, ".slice")) {
-		return errors.New("conmon cgroup should be 'pod' or a systemd slice")
+	if err := c.ValidateDefaultRuntime(); err != nil {
+		return err
 	}
 
 	if c.LogSizeMax >= 0 && c.LogSizeMax < OCIBufSize {
@@ -918,39 +998,46 @@ func (c *RuntimeConfig) Validate(systemContext *types.SystemContext, onExecution
 	}
 
 	if _, err := c.Sysctls(); err != nil {
-		return errors.Wrap(err, "invalid default_sysctls")
+		return fmt.Errorf("invalid default_sysctls: %w", err)
 	}
 
 	if err := c.DefaultCapabilities.Validate(); err != nil {
-		return errors.Wrapf(err, "invalid capabilities")
+		return fmt.Errorf("invalid capabilities: %w", err)
 	}
 
 	if c.InfraCtrCPUSet != "" {
 		set, err := cpuset.Parse(c.InfraCtrCPUSet)
 		if err != nil {
-			return errors.Wrap(err, "invalid infra_ctr_cpuset")
+			return fmt.Errorf("invalid infra_ctr_cpuset: %w", err)
 		}
 
 		executable, err := exec.LookPath(tasksetBinary)
 		if err != nil {
-			return errors.Wrapf(err, "%q not found in $PATH", tasksetBinary)
+			return fmt.Errorf("%q not found in $PATH: %w", tasksetBinary, err)
 		}
 		cmdrunner.PrependCommandsWith(executable, "--cpu-list", set.String())
 	}
 
 	if err := c.Workloads.Validate(); err != nil {
-		return errors.Wrap(err, "workloads validation")
+		return fmt.Errorf("workloads validation: %w", err)
 	}
 
 	// check for validation on execution
 	if onExecution {
+		// First, configure cgroup manager so the values of the Runtime.MonitorCgroup can be validated
+		cgroupManager, err := cgmgr.SetCgroupManager(c.CgroupManagerName)
+		if err != nil {
+			return fmt.Errorf("unable to update cgroup manager: %w", err)
+		}
+		c.cgroupManager = cgroupManager
+
 		if err := c.ValidateRuntimes(); err != nil {
-			return errors.Wrap(err, "runtime validation")
+			return fmt.Errorf("runtime validation: %w", err)
 		}
 
 		// Validate the system registries configuration
 		if _, err := sysregistriesv2.GetRegistries(systemContext); err != nil {
-			return errors.Wrap(err, "invalid registries")
+			return fmt.Errorf("invalid registries: %w", err)
 		}
 
 		// we should use a hooks directory if
@@ -975,55 +1062,100 @@ func (c *RuntimeConfig) Validate(systemContext *types.SystemContext, onExecution
 		}
 		c.HooksDir = hooksDirs
 
-		// Validate the conmon path
-		if err := c.ValidateConmonPath("conmon"); err != nil {
-			return errors.Wrap(err, "conmon validation")
-		}
+		cdi.GetRegistry(cdi.WithSpecDirs(c.CDISpecDirs...))
 
 		// Validate the pinns path
 		if err := c.ValidatePinnsPath("pinns"); err != nil {
-			return errors.Wrap(err, "pinns validation")
+			return fmt.Errorf("pinns validation: %w", err)
 		}
 
 		c.namespaceManager = nsmgr.New(c.NamespacesDir, c.PinnsPath)
 		if err := c.namespaceManager.Initialize(); err != nil {
-			return errors.Wrapf(err, "initialize nsmgr")
+			return fmt.Errorf("initialize nsmgr: %w", err)
 		}
 
-		if c.SeccompUseDefaultWhenEmpty {
-			c.seccompConfig.SetDefaultWhenEmpty()
+		if c.EnableCriuSupport {
+			if err := validateCriuInPath(); err != nil {
+				c.EnableCriuSupport = false
+				return errors.New("cannot enable checkpoint/restore support without the criu binary in $PATH")
+			}
+			logrus.Infof("Checkpoint/restore support enabled")
+		} else {
+			logrus.Infof("Checkpoint/restore support disabled")
 		}
 
+		c.seccompConfig.SetUseDefaultWhenEmpty(c.SeccompUseDefaultWhenEmpty)
+
 		if err := c.seccompConfig.LoadProfile(c.SeccompProfile); err != nil {
-			return errors.Wrap(err, "unable to load seccomp profile")
+			if !errors.Is(err, os.ErrNotExist) {
+				return fmt.Errorf("unable to load seccomp profile: %w", err)
+			}
+
+			logrus.Info("Specified profile does not exist on disk")
+			if err := c.seccompConfig.LoadDefaultProfile(); err != nil {
+				return fmt.Errorf("load default seccomp profile: %w", err)
+			}
 		}
 
 		if err := c.apparmorConfig.LoadProfile(c.ApparmorProfile); err != nil {
-			return errors.Wrap(err, "unable to load AppArmor profile")
+			return fmt.Errorf("unable to load AppArmor profile: %w", err)
 		}
 
 		if err := c.blockioConfig.Load(c.BlockIOConfigFile); err != nil {
-			return errors.Wrap(err, "blockio configuration")
+			return fmt.Errorf("blockio configuration: %w", err)
 		}
 
 		if err := c.rdtConfig.Load(c.RdtConfigFile); err != nil {
-			return errors.Wrap(err, "rdt configuration")
-		}
-
-		cgroupManager, err := cgmgr.SetCgroupManager(c.CgroupManagerName)
-		if err != nil {
-			return errors.Wrap(err, "unable to update cgroup manager")
-		}
-		c.cgroupManager = cgroupManager
-
-		if !c.cgroupManager.IsSystemd() && c.ConmonCgroup != "pod" && c.ConmonCgroup != "" {
-			return errors.New("cgroupfs manager conmon cgroup should be 'pod' or empty")
+			return fmt.Errorf("rdt configuration: %w", err)
 		}
 	}
 
+	if err := c.TranslateMonitorFields(onExecution); err != nil {
+		return fmt.Errorf("monitor fields translation: %w", err)
+	}
+
 	return nil
 }
 
+// ValidateDefaultRuntime ensures that the default runtime is set and valid.
+func (c *RuntimeConfig) ValidateDefaultRuntime() error {
+	// If the default runtime is defined in the runtime entry table, then it is valid
+	if _, ok := c.Runtimes[c.DefaultRuntime]; ok {
+		return nil
+	}
+
+	// If a non-empty runtime does not exist in the runtime entry table, this is an error.
+	if c.DefaultRuntime != "" {
+		return fmt.Errorf("default_runtime set to %q, but no runtime entry table [crio.runtime.runtimes.%s] was found", c.DefaultRuntime, c.DefaultRuntime)
+	}
+
+	// Set the default runtime to "runc" if default_runtime is not set
+	logrus.Debugf("Defaulting to %q as the runtime since default_runtime is not set", defaultRuntime)
+	// The default config sets runc and its path in the runtimes map, so check for that
+	// first. If it does not exist then we add runc + its path to the runtimes map.
+	if _, ok := c.Runtimes[defaultRuntime]; !ok {
+		c.Runtimes[defaultRuntime] = defaultRuntimeHandler()
+	}
+	// Set the DefaultRuntime to runc so we don't fail further along in the code
+	c.DefaultRuntime = defaultRuntime
+
+	return nil
+}
+
+func defaultRuntimeHandler() *RuntimeHandler {
+	return &RuntimeHandler{
+		RuntimeType: DefaultRuntimeType,
+		RuntimeRoot: DefaultRuntimeRoot,
+		AllowedAnnotations: []string{
+			annotations.OCISeccompBPFHookAnnotation,
+		},
+		MonitorEnv: []string{
+			"PATH=/usr/local/sbin:/usr/local/bin:/usr/sbin:/usr/bin:/sbin:/bin",
+		},
+		MonitorCgroup: defaultMonitorCgroup,
+	}
+}
+
 // ValidateRuntimes checks every runtime if its members are valid
 func (c *RuntimeConfig) ValidateRuntimes() error {
 	var failedValidation []string
@@ -1047,16 +1179,62 @@ func (c *RuntimeConfig) ValidateRuntimes() error {
 	return nil
 }
 
+func (c *RuntimeConfig) TranslateMonitorFields(onExecution bool) error {
+	for name, handler := range c.Runtimes {
+		if handler.RuntimeType == DefaultRuntimeType || handler.RuntimeType == "" {
+			if err := c.TranslateMonitorFieldsForHandler(handler, onExecution); err != nil {
+				return fmt.Errorf("failed to translate monitor fields for runtime %s: %w", name, err)
+			}
+		}
+	}
+	return nil
+}
+
+// TranslateMonitorFields is a transitional function that takes the configuration fields
+// previously held by the RuntimeConfig that are being moved inside of the runtime handler structure.
+func (c *RuntimeConfig) TranslateMonitorFieldsForHandler(handler *RuntimeHandler, onExecution bool) error {
+	if c.ConmonCgroup != "" {
+		logrus.Debugf("Monitor cgroup %s is becoming %s", handler.MonitorCgroup, c.ConmonCgroup)
+		handler.MonitorCgroup = c.ConmonCgroup
+	}
+	if c.Conmon != "" {
+		logrus.Debugf("Monitor path %s is becoming %s", handler.MonitorPath, c.Conmon)
+		handler.MonitorPath = c.Conmon
+	}
+	if len(c.ConmonEnv) != 0 {
+		handler.MonitorEnv = c.ConmonEnv
+	}
+	// If empty, assume default
+	if handler.MonitorCgroup == "" {
+		handler.MonitorCgroup = defaultMonitorCgroup
+	}
+	if onExecution {
+		if err := c.ValidateConmonPath("conmon", handler); err != nil {
+			return err
+		}
+		if !c.cgroupManager.IsSystemd() {
+			if handler.MonitorCgroup != utils.PodCgroupName && handler.MonitorCgroup != "" {
+				return errors.New("cgroupfs manager conmon cgroup should be 'pod' or empty")
+			}
+			return nil
+		}
+		if !(handler.MonitorCgroup == utils.PodCgroupName || strings.HasSuffix(handler.MonitorCgroup, ".slice")) {
+			return errors.New("conmon cgroup should be 'pod' or a systemd slice")
+		}
+	}
+	return nil
+}
+
 // ValidateConmonPath checks if `Conmon` is set within the `RuntimeConfig`.
 // If this is not the case, it tries to find it within the $PATH variable.
 // In any other case, it simply checks if `Conmon` is a valid file.
-func (c *RuntimeConfig) ValidateConmonPath(executable string) error {
+func (c *RuntimeConfig) ValidateConmonPath(executable string, handler *RuntimeHandler) error {
 	var err error
-	c.Conmon, err = validateExecutablePath(executable, c.Conmon)
+	handler.MonitorPath, err = validateExecutablePath(executable, handler.MonitorPath)
 	if err != nil {
 		return err
 	}
-	c.conmonManager, err = conmonmgr.New(c.Conmon)
+	c.conmonManager, err = conmonmgr.New(handler.MonitorPath)
 
 	return err
 }
@@ -1076,6 +1254,12 @@ func (c *RuntimeConfig) ValidatePinnsPath(executable string) error {
 	return err
 }
 
+func validateCriuInPath() error {
+	_, err := validateExecutablePath("criu", "")
+
+	return err
+}
+
 // Seccomp returns the seccomp configuration
 func (c *RuntimeConfig) Seccomp() *seccomp.Config {
 	return c.seccompConfig
@@ -1115,6 +1299,10 @@ func (c *RuntimeConfig) Devices() []device.Device {
 	return c.deviceConfig.Devices()
 }
 
+func (c *RuntimeConfig) CheckpointRestore() bool {
+	return c.EnableCriuSupport
+}
+
 func validateExecutablePath(executable, currentPath string) (string, error) {
 	if currentPath == "" {
 		path, err := exec.LookPath(executable)
@@ -1125,7 +1313,7 @@ func validateExecutablePath(executable, currentPath string) (string, error) {
 		return path, nil
 	}
 	if _, err := os.Stat(currentPath); err != nil {
-		return "", errors.Wrapf(err, "invalid %s path", executable)
+		return "", fmt.Errorf("invalid %s path: %w", executable, err)
 	}
 	logrus.Infof("Using %s executable: %s", executable, currentPath)
 	return currentPath, nil
@@ -1141,23 +1329,23 @@ func (c *NetworkConfig) Validate(onExecution bool) error {
 		if err != nil {
 			if os.IsNotExist(err) {
 				if err = os.MkdirAll(c.NetworkDir, 0o755); err != nil {
-					return errors.Wrapf(err, "Cannot create network_dir: %s", c.NetworkDir)
+					return fmt.Errorf("cannot create network_dir: %s: %w", c.NetworkDir, err)
 				}
 			} else {
-				return errors.Wrapf(err, "invalid network_dir: %s", c.NetworkDir)
+				return fmt.Errorf("invalid network_dir: %s: %w", c.NetworkDir, err)
 			}
 		}
 
 		for _, pluginDir := range c.PluginDirs {
 			if err := os.MkdirAll(pluginDir, 0o755); err != nil {
-				return errors.Wrap(err, "invalid plugin_dirs entry")
+				return fmt.Errorf("invalid plugin_dirs entry: %w", err)
 			}
 		}
 		// While the plugin_dir option is being deprecated, we need this check
 		if c.PluginDir != "" {
 			logrus.Warnf("The config field plugin_dir is being deprecated. Please use plugin_dirs instead")
 			if err := os.MkdirAll(c.PluginDir, 0o755); err != nil {
-				return errors.Wrap(err, "invalid plugin_dir entry")
+				return fmt.Errorf("invalid plugin_dir entry: %w", err)
 			}
 			// Append PluginDir to PluginDirs, so from now on we can operate in terms of PluginDirs and not worry
 			// about missing cases.
@@ -1173,7 +1361,7 @@ func (c *NetworkConfig) Validate(onExecution bool) error {
 			c.CNIDefaultNetwork, c.NetworkDir, c.PluginDirs...,
 		)
 		if err != nil {
-			return errors.Wrap(err, "initialize CNI plugin")
+			return fmt.Errorf("initialize CNI plugin: %w", err)
 		}
 		c.cniManager = cniManager
 	}
@@ -1217,7 +1405,7 @@ func (r *RuntimeHandler) ValidateRuntimePath(name string) error {
 	if r.RuntimePath == "" {
 		executable, err := exec.LookPath(name)
 		if err != nil {
-			return errors.Wrapf(err, "%q not found in $PATH", name)
+			return fmt.Errorf("%q not found in $PATH: %w", name, err)
 		}
 		r.RuntimePath = executable
 		logrus.Debugf("Using runtime executable from $PATH %q", executable)
@@ -1240,8 +1428,8 @@ func (r *RuntimeHandler) ValidateRuntimePath(name string) error {
 
 // ValidateRuntimeType checks if the `RuntimeType` is valid.
 func (r *RuntimeHandler) ValidateRuntimeType(name string) error {
-	if r.RuntimeType != "" && r.RuntimeType != DefaultRuntimeType && r.RuntimeType != RuntimeTypeVM {
-		return errors.Errorf("invalid `runtime_type` %q for runtime %q",
+	if r.RuntimeType != "" && r.RuntimeType != DefaultRuntimeType && r.RuntimeType != RuntimeTypeVM && r.RuntimeType != RuntimeTypePod {
+		return fmt.Errorf("invalid `runtime_type` %q for runtime %q",
 			r.RuntimeType, name)
 	}
 	return nil
@@ -1263,25 +1451,35 @@ func (r *RuntimeHandler) ValidateRuntimeConfigPath(name string) error {
 }
 
 func (r *RuntimeHandler) ValidateRuntimeAllowedAnnotations() error {
-	disallowedAnnotations := make(map[string]struct{})
-	for _, ann := range annotations.AllAllowedAnnotations {
-		disallowedAnnotations[ann] = struct{}{}
-	}
-	for _, allowed := range r.AllowedAnnotations {
-		if _, ok := disallowedAnnotations[allowed]; !ok {
-			return errors.Errorf("invalid allowed_annotation: %s", allowed)
-		}
-		delete(disallowedAnnotations, allowed)
-	}
-	for ann := range disallowedAnnotations {
-		r.DisallowedAnnotations = append(r.DisallowedAnnotations, ann)
+	disallowed, err := validateAllowedAndGenerateDisallowedAnnotations(r.AllowedAnnotations)
+	if err != nil {
+		return err
 	}
 	logrus.Debugf(
 		"Allowed annotations for runtime: %v", r.AllowedAnnotations,
 	)
+	r.DisallowedAnnotations = disallowed
 	return nil
 }
 
+func validateAllowedAndGenerateDisallowedAnnotations(allowed []string) (disallowed []string, _ error) {
+	disallowedMap := make(map[string]struct{})
+	for _, ann := range annotations.AllAllowedAnnotations {
+		disallowedMap[ann] = struct{}{}
+	}
+	for _, ann := range allowed {
+		if _, ok := disallowedMap[ann]; !ok {
+			return nil, fmt.Errorf("invalid allowed_annotation: %s", ann)
+		}
+		delete(disallowedMap, ann)
+	}
+	disallowed = make([]string, 0, len(disallowedMap))
+	for ann := range disallowedMap {
+		disallowed = append(disallowed, ann)
+	}
+	return disallowed, nil
+}
+
 // CNIPlugin returns the network configuration CNI plugin
 func (c *NetworkConfig) CNIPlugin() ocicni.CNIPlugin {
 	return c.cniManager.Plugin()
