@@ -45,6 +45,8 @@ func assembleTemplateString(displayAllConfig bool, c *Config) string {
 
 	// [crio.tracing] configuration
 	templateString += crioTemplateString(crioTracingConfig, templateStringCrioTracing, displayAllConfig, crioTemplateConfig)
+	// [crio.nri] configuration
+	templateString += crioTemplateString(crioNRIConfig, templateStringCrioNRI, displayAllConfig, crioTemplateConfig)
 
 	// [crio.stats] configuration
 	templateString += crioTemplateString(crioStatsConfig, templateStringCrioStats, displayAllConfig, crioTemplateConfig)
@@ -87,6 +89,7 @@ const (
 	crioMetricsConfig
 	crioTracingConfig
 	crioStatsConfig
+	crioNRIConfig
 )
 
 type templateConfigValue struct {
@@ -417,6 +420,11 @@ func initCrioTemplateConfig(c *Config) ([]*templateConfigValue, error) {
 			group:          crioRuntimeConfig,
 			isDefaultValue: simpleEqual(dc.EnableCriuSupport, c.EnableCriuSupport),
 		},
+		{
+			templateString: templateStringCrioRuntimeEnablePodEvents,
+			group:          crioRuntimeConfig,
+			isDefaultValue: simpleEqual(dc.EnablePodEvents, c.EnablePodEvents),
+		},
 		{
 			templateString: templateStringCrioRuntimeDefaultRuntime,
 			group:          crioRuntimeConfig,
@@ -547,6 +555,41 @@ func initCrioTemplateConfig(c *Config) ([]*templateConfigValue, error) {
 			group:          crioStatsConfig,
 			isDefaultValue: simpleEqual(dc.StatsCollectionPeriod, c.StatsCollectionPeriod),
 		},
+		{
+			templateString: templateStringCrioNRIEnable,
+			group:          crioNRIConfig,
+			isDefaultValue: simpleEqual(dc.NRI.Enabled, c.NRI.Enabled),
+		},
+		{
+			templateString: templateStringCrioNRISocketPath,
+			group:          crioNRIConfig,
+			isDefaultValue: simpleEqual(dc.NRI.SocketPath, c.NRI.SocketPath),
+		},
+		{
+			templateString: templateStringCrioNRIPluginDir,
+			group:          crioNRIConfig,
+			isDefaultValue: simpleEqual(dc.NRI.PluginPath, c.NRI.PluginPath),
+		},
+		{
+			templateString: templateStringCrioNRIPluginConfigDir,
+			group:          crioNRIConfig,
+			isDefaultValue: simpleEqual(dc.NRI.PluginPath, c.NRI.PluginPath),
+		},
+		{
+			templateString: templateStringCrioNRIDisableConnections,
+			group:          crioNRIConfig,
+			isDefaultValue: simpleEqual(dc.NRI.DisableConnections, c.NRI.DisableConnections),
+		},
+		{
+			templateString: templateStringCrioNRIPluginRegistrationTimeout,
+			group:          crioNRIConfig,
+			isDefaultValue: simpleEqual(dc.NRI.PluginRegistrationTimeout, c.NRI.PluginRegistrationTimeout),
+		},
+		{
+			templateString: templateStringCrioNRIPluginRequestTimeout,
+			group:          crioNRIConfig,
+			isDefaultValue: simpleEqual(dc.NRI.PluginRequestTimeout, c.NRI.PluginRequestTimeout),
+		},
 	}
 
 	return crioTemplateConfig, nil
@@ -1062,6 +1105,12 @@ const templateStringCrioRuntimeEnableCriuSupport = `# Globally enable/disable CR
 
 `
 
+const templateStringCrioRuntimeEnablePodEvents = `# Enable/disable the generation of the container,
+# sandbox lifecycle events to be sent to the Kubelet to optimize the PLEG
+{{ $.Comment }}enable_pod_events = {{ .EnablePodEvents }}
+
+`
+
 const templateStringCrioRuntimeDefaultRuntime = `# default_runtime is the _name_ of the OCI runtime to be used as the default.
 # default_runtime is the _name_ of the OCI runtime to be used as the default.
 # The name is matched against the runtimes map below.
@@ -1127,6 +1176,34 @@ const templateStringCrioRuntimeRuntimesRuntimeHandler = `# The "crio.runtime.run
 #   should be moved to the container's cgroup
 # - monitor_env (optional, array of strings): Environment variables to pass to the montior.
 #   Replaces deprecated option "conmon_env".
+#
+# Using the seccomp notifier feature:
+#
+# This feature can help you to debug seccomp related issues, for example if
+# blocked syscalls (permission denied errors) have negative impact on the workload.
+#
+# To be able to use this feature, configure a runtime which has the annotation
+# "io.kubernetes.cri-o.seccompNotifierAction" in the allowed_annotations array.
+#
+# It also requires at least runc 1.1.0 or crun 0.19 which support the notifier
+# feature.
+#
+# If everything is setup, CRI-O will modify chosen seccomp profiles for
+# containers if the annotation "io.kubernetes.cri-o.seccompNotifierAction" is
+# set on the Pod sandbox. CRI-O will then get notified if a container is using
+# a blocked syscall and then terminate the workload after a timeout of 5
+# seconds if the value of "io.kubernetes.cri-o.seccompNotifierAction=stop".
+#
+# This also means that multiple syscalls can be captured during that period,
+# while the timeout will get reset once a new syscall has been discovered.
+#
+# This also means that the Pods "restartPolicy" has to be set to "Never",
+# otherwise the kubelet will restart the container immediately.
+#
+# Please be aware that CRI-O is not able to get notified if a syscall gets
+# blocked based on the seccomp defaultAction, which is a general runtime
+# limitation.
+
 {{ range $runtime_name, $runtime_handler := .Runtimes  }}
 {{ $.Comment }}[crio.runtime.runtimes.{{ $runtime_name }}]
 {{ $.Comment }}runtime_path = "{{ $runtime_handler.RuntimePath }}"
@@ -1172,9 +1249,9 @@ const templateStringCrioRuntimeWorkloads = `# The workloads table defines ways t
 {{ $.Comment }}[crio.runtime.workloads.{{ $workload_type }}]
 {{ $.Comment }}activation_annotation = "{{ $workload_config.ActivationAnnotation }}"
 {{ $.Comment }}annotation_prefix = "{{ $workload_config.AnnotationPrefix }}"
-{{ $.Comment }}[crio.runtime.workloads.{{ $workload_type }}.resources]
+{{ if $workload_config.Resources }}{{ $.Comment }}[crio.runtime.workloads.{{ $workload_type }}.resources]
 {{ $.Comment }}cpuset = "{{ $workload_config.Resources.CPUSet }}"
-{{ $.Comment }}cpushares = {{ $workload_config.Resources.CPUShares }}
+{{ $.Comment }}cpushares = {{ $workload_config.Resources.CPUShares }}{{ end }}
 {{ end }}
 `
 
@@ -1330,7 +1407,7 @@ const templateStringCrioTracingTracingEndpoint = `# Address on which the gRPC tr
 
 `
 
-const templateStringCrioTracingTracingSamplingRatePerMillion = `# Number of samples to collect per million spans.
+const templateStringCrioTracingTracingSamplingRatePerMillion = `# Number of samples to collect per million spans. Set to 1000000 to always sample.
 {{ $.Comment }}tracing_sampling_rate_per_million = {{ .TracingSamplingRatePerMillion }}
 
 `
@@ -1345,3 +1422,43 @@ const templateStringCrioStatsStatsCollectionPeriod = `# The number of seconds be
 {{ $.Comment }}stats_collection_period = {{ .StatsCollectionPeriod }}
 
 `
+
+const templateStringCrioNRI = `# CRI-O NRI configuration.
+[crio.nri]
+
+`
+
+const templateStringCrioNRIEnable = `# Globally enable or disable NRI.
+{{ $.Comment }}enable_nri = {{ .NRI.Enabled }}
+
+`
+
+const templateStringCrioNRISocketPath = `# NRI socket to listen on.
+{{ $.Comment }}nri_listen = "{{ .NRI.SocketPath }}"
+
+`
+
+const templateStringCrioNRIPluginDir = `# NRI plugin directory to use.
+{{ $.Comment }}nri_plugin_dir = "{{ .NRI.PluginPath }}"
+
+`
+
+const templateStringCrioNRIPluginConfigDir = `# NRI plugin configuration directory to use.
+{{ $.Comment }}nri_plugin_config_dir = "{{ .NRI.PluginConfigPath }}"
+
+`
+
+const templateStringCrioNRIDisableConnections = `# Disable connections from externally launched NRI plugins.
+{{ $.Comment }}nri_disable_connections = {{ .NRI.DisableConnections }}
+
+`
+
+const templateStringCrioNRIPluginRegistrationTimeout = `# Timeout for a plugin to register itself with NRI.
+{{ $.Comment }}nri_plugin_registration_timeout = "{{ .NRI.PluginRegistrationTimeout }}"
+
+`
+
+const templateStringCrioNRIPluginRequestTimeout = `# Timeout for a plugin to handle an NRI request.
+{{ $.Comment }}nri_plugin_request_timeout = "{{ .NRI.PluginRequestTimeout }}"
+
+`
