@@ -30,12 +30,14 @@ var _ = t.Describe("ContainerRemove", func() {
 				State: specs.State{Status: oci.ContainerStateStopped},
 			})
 			gomock.InOrder(
-				runtimeServerMock.EXPECT().DeleteContainer(gomock.Any()).
+				runtimeServerMock.EXPECT().StopContainer(gomock.Any(), gomock.Any()).
+					Return(nil),
+				runtimeServerMock.EXPECT().DeleteContainer(gomock.Any(), gomock.Any()).
 					Return(nil),
 			)
 
 			// When
-			err := sut.RemoveContainer(context.Background(),
+			_, err := sut.RemoveContainer(context.Background(),
 				&types.RemoveContainerRequest{
 					ContainerId: testContainer.ID(),
 				})
@@ -47,7 +49,7 @@ var _ = t.Describe("ContainerRemove", func() {
 		It("should fail on container remove error", func() {
 			// Given
 			// When
-			err := sut.RemoveContainer(context.Background(),
+			_, err := sut.RemoveContainer(context.Background(),
 				&types.RemoveContainerRequest{})
 
 			// Then
