@@ -3,29 +3,33 @@ package server
 import (
 	"fmt"
 
+	"github.com/cri-o/cri-o/internal/log"
 	"github.com/cri-o/cri-o/internal/oci"
 	"golang.org/x/net/context"
 	types "k8s.io/cri-api/pkg/apis/runtime/v1"
 )
 
 // ReopenContainerLog reopens the containers log file
-func (s *Server) ReopenContainerLog(ctx context.Context, req *types.ReopenContainerLogRequest) error {
-	c, err := s.GetContainerFromShortID(req.ContainerId)
+func (s *Server) ReopenContainerLog(ctx context.Context, req *types.ReopenContainerLogRequest) (*types.ReopenContainerLogResponse, error) {
+	ctx, span := log.StartSpan(ctx)
+	defer span.End()
+	c, err := s.GetContainerFromShortID(ctx, req.ContainerId)
 	if err != nil {
-		return fmt.Errorf("could not find container %s: %w", req.ContainerId, err)
+		return nil, fmt.Errorf("could not find container %s: %w", req.ContainerId, err)
 	}
 
 	if err := s.ContainerServer.Runtime().UpdateContainerStatus(ctx, c); err != nil {
-		return err
+		return nil, err
 	}
 
 	cState := c.State()
 	if !(cState.Status == oci.ContainerStateRunning || cState.Status == oci.ContainerStateCreated) {
-		return fmt.Errorf("container is not created or running")
+		return nil, fmt.Errorf("container is not created or running")
 	}
 
 	if err := s.ContainerServer.Runtime().ReopenContainerLog(ctx, c); err != nil {
-		return err
+		return nil, err
 	}
-	return nil
+
+	return &types.ReopenContainerLogResponse{}, nil
 }
