@@ -10,20 +10,28 @@ import (
 )
 
 // RemoveImage removes the image.
-func (s *Server) RemoveImage(ctx context.Context, req *types.RemoveImageRequest) error {
+func (s *Server) RemoveImage(ctx context.Context, req *types.RemoveImageRequest) (*types.RemoveImageResponse, error) {
+	ctx, span := log.StartSpan(ctx)
+	defer span.End()
 	imageRef := ""
 	img := req.Image
 	if img != nil {
 		imageRef = img.Image
 	}
 	if imageRef == "" {
-		return fmt.Errorf("no image specified")
+		return nil, fmt.Errorf("no image specified")
 	}
-	return s.removeImage(ctx, imageRef)
+	if err := s.removeImage(ctx, imageRef); err != nil {
+		return nil, err
+	}
+	return &types.RemoveImageResponse{}, nil
 }
 
 func (s *Server) removeImage(ctx context.Context, imageRef string) error {
 	var deleted bool
+	ctx, span := log.StartSpan(ctx)
+	defer span.End()
+
 	images, err := s.StorageImageServer().ResolveNames(s.config.SystemContext, imageRef)
 	if err != nil {
 		if err == storage.ErrCannotParseImageID {
