@@ -1,6 +1,7 @@
 package metrics
 
 import (
+	"context"
 	"crypto/tls"
 	"crypto/x509"
 	"errors"
@@ -14,7 +15,7 @@ import (
 
 	"github.com/cri-o/cri-o/internal/process"
 	libconfig "github.com/cri-o/cri-o/pkg/config"
-	"github.com/cri-o/cri-o/server/metrics/collectors"
+	"github.com/cri-o/cri-o/server/otel-collector/collectors"
 	"github.com/fsnotify/fsnotify"
 	"github.com/prometheus/client_golang/prometheus"
 	"github.com/prometheus/client_golang/prometheus/promhttp"
@@ -62,31 +63,32 @@ func GetSizeBucket(size float64) string {
 
 // Metrics is the main structure for starting the metrics endpoints.
 type Metrics struct {
-	config                              *libconfig.MetricsConfig
-	metricOperations                    *prometheus.CounterVec // Deprecated: in favour of metricOperationsTotal
-	metricOperationsLatency             *prometheus.GaugeVec   // Deprecated: in favour of metricOperationsLatencySeconds
-	metricOperationsLatencyTotal        *prometheus.SummaryVec // Deprecated: in favour of metricOperationsLatencySecondsTotal
-	metricOperationsErrors              *prometheus.CounterVec // Deprecated: in favour of metricOperationsErrorsTotal
-	metricImagePullsByDigest            *prometheus.CounterVec // Deprecated: in favour of metricImagePullsBytesTotal
-	metricImagePullsByName              *prometheus.CounterVec // Deprecated: in favour of metricImagePullsBytesTotal
-	metricImagePullsByNameSkipped       *prometheus.CounterVec // Deprecated: in favour of metricImagePullsSkippedBytesTotal
-	metricImagePullsFailures            *prometheus.CounterVec // Deprecated: in favour of metricImagePullsFailureTotal
-	metricImagePullsSuccesses           *prometheus.CounterVec // Deprecated: in favour of metricImagePullsSuccessTotal
-	metricImagePullsLayerSize           prometheus.Histogram
-	metricImageLayerReuse               *prometheus.CounterVec // Deprecated: in favour of metricImageLayerReuseTotal
-	metricContainersOOMTotal            prometheus.Counter
-	metricContainersOOM                 *prometheus.CounterVec // Deprecated: in favour of metricContainersOOMCountTotal
-	metricProcessesDefunct              prometheus.GaugeFunc
-	metricOperationsTotal               *prometheus.CounterVec
-	metricOperationsLatencySeconds      *prometheus.GaugeVec
-	metricOperationsLatencySecondsTotal *prometheus.SummaryVec
-	metricOperationsErrorsTotal         *prometheus.CounterVec
-	metricImagePullsBytesTotal          *prometheus.CounterVec
-	metricImagePullsSkippedBytesTotal   *prometheus.CounterVec
-	metricImagePullsFailureTotal        *prometheus.CounterVec
-	metricImagePullsSuccessTotal        prometheus.Counter
-	metricImageLayerReuseTotal          *prometheus.CounterVec
-	metricContainersOOMCountTotal       *prometheus.CounterVec
+	config                                    *libconfig.MetricsConfig
+	metricOperations                          *prometheus.CounterVec // Deprecated: in favour of metricOperationsTotal
+	metricOperationsLatency                   *prometheus.GaugeVec   // Deprecated: in favour of metricOperationsLatencySeconds
+	metricOperationsLatencyTotal              *prometheus.SummaryVec // Deprecated: in favour of metricOperationsLatencySecondsTotal
+	metricOperationsErrors                    *prometheus.CounterVec // Deprecated: in favour of metricOperationsErrorsTotal
+	metricImagePullsByDigest                  *prometheus.CounterVec // Deprecated: in favour of metricImagePullsBytesTotal
+	metricImagePullsByName                    *prometheus.CounterVec // Deprecated: in favour of metricImagePullsBytesTotal
+	metricImagePullsByNameSkipped             *prometheus.CounterVec // Deprecated: in favour of metricImagePullsSkippedBytesTotal
+	metricImagePullsFailures                  *prometheus.CounterVec // Deprecated: in favour of metricImagePullsFailureTotal
+	metricImagePullsSuccesses                 *prometheus.CounterVec // Deprecated: in favour of metricImagePullsSuccessTotal
+	metricImagePullsLayerSize                 prometheus.Histogram
+	metricImageLayerReuse                     *prometheus.CounterVec // Deprecated: in favour of metricImageLayerReuseTotal
+	metricContainersOOMTotal                  prometheus.Counter
+	metricContainersOOM                       *prometheus.CounterVec // Deprecated: in favour of metricContainersOOMCountTotal
+	metricProcessesDefunct                    prometheus.GaugeFunc
+	metricOperationsTotal                     *prometheus.CounterVec
+	metricOperationsLatencySeconds            *prometheus.GaugeVec
+	metricOperationsLatencySecondsTotal       *prometheus.SummaryVec
+	metricOperationsErrorsTotal               *prometheus.CounterVec
+	metricImagePullsBytesTotal                *prometheus.CounterVec
+	metricImagePullsSkippedBytesTotal         *prometheus.CounterVec
+	metricImagePullsFailureTotal              *prometheus.CounterVec
+	metricImagePullsSuccessTotal              prometheus.Counter
+	metricImageLayerReuseTotal                *prometheus.CounterVec
+	metricContainersOOMCountTotal             *prometheus.CounterVec
+	metricContainersSeccompNotifierCountTotal *prometheus.CounterVec
 }
 
 var instance *Metrics
@@ -309,6 +311,14 @@ func New(config *libconfig.MetricsConfig) *Metrics {
 			// keeping the label cardinality of `name` reasonably low.
 			[]string{"name"},
 		),
+		metricContainersSeccompNotifierCountTotal: prometheus.NewCounterVec(
+			prometheus.CounterOpts{
+				Subsystem: collectors.Subsystem,
+				Name:      collectors.ContainersSeccompNotifierCountTotal.String(),
+				Help:      "Number of forbidden syscalls by syscall and container name",
+			},
+			[]string{"name", "syscall"},
+		),
 	}
 	return Instance()
 }
@@ -439,6 +449,15 @@ func (m *Metrics) MetricContainersOOMTotalInc() {
 	m.metricContainersOOMTotal.Inc()
 }
 
+func (m *Metrics) MetricContainersSeccompNotifierCountTotalInc(name, syscall string) {
+	c, err := m.metricContainersSeccompNotifierCountTotal.GetMetricWithLabelValues(name, syscall)
+	if err != nil {
+		logrus.Warnf("Unable to write container seccomp notifier metric: %v", err)
+		return
+	}
+	c.Inc()
+}
+
 func (m *Metrics) MetricImagePullsLayerSizeObserve(size int64) {
 	m.metricImagePullsLayerSize.Observe(float64(size))
 }
@@ -549,16 +568,17 @@ func (m *Metrics) createEndpoint() (*http.ServeMux, error) {
 		collectors.ContainersOOM:           m.metricContainersOOM,
 		collectors.ProcessesDefunct:        m.metricProcessesDefunct,
 
-		collectors.OperationsTotal:               m.metricOperationsTotal,
-		collectors.OperationsLatencySeconds:      m.metricOperationsLatencySeconds,
-		collectors.OperationsLatencySecondsTotal: m.metricOperationsLatencySecondsTotal,
-		collectors.OperationsErrorsTotal:         m.metricOperationsErrorsTotal,
-		collectors.ImagePullsBytesTotal:          m.metricImagePullsBytesTotal,
-		collectors.ImagePullsSkippedBytesTotal:   m.metricImagePullsSkippedBytesTotal,
-		collectors.ImagePullsFailureTotal:        m.metricImagePullsFailureTotal,
-		collectors.ImagePullsSuccessTotal:        m.metricImagePullsSuccessTotal,
-		collectors.ImageLayerReuseTotal:          m.metricImageLayerReuseTotal,
-		collectors.ContainersOOMCountTotal:       m.metricContainersOOMCountTotal,
+		collectors.OperationsTotal:                     m.metricOperationsTotal,
+		collectors.OperationsLatencySeconds:            m.metricOperationsLatencySeconds,
+		collectors.OperationsLatencySecondsTotal:       m.metricOperationsLatencySecondsTotal,
+		collectors.OperationsErrorsTotal:               m.metricOperationsErrorsTotal,
+		collectors.ImagePullsBytesTotal:                m.metricImagePullsBytesTotal,
+		collectors.ImagePullsSkippedBytesTotal:         m.metricImagePullsSkippedBytesTotal,
+		collectors.ImagePullsFailureTotal:              m.metricImagePullsFailureTotal,
+		collectors.ImagePullsSuccessTotal:              m.metricImagePullsSuccessTotal,
+		collectors.ImageLayerReuseTotal:                m.metricImageLayerReuseTotal,
+		collectors.ContainersOOMCountTotal:             m.metricContainersOOMCountTotal,
+		collectors.ContainersSeccompNotifierCountTotal: m.metricContainersSeccompNotifierCountTotal,
 	} {
 		if m.config.MetricsCollectors.Contains(collector) {
 			logrus.Debugf("Enabling metric: %s", collector.Stripped())
@@ -585,6 +605,11 @@ func (m *Metrics) startEndpoint(
 
 	go func() {
 		var err error
+
+		srv := http.Server{
+			Handler: me,
+		}
+
 		if m.config.MetricsCert != "" && m.config.MetricsKey != "" {
 			logrus.Infof("Serving metrics on %s via HTTPs", address)
 
@@ -595,21 +620,31 @@ func (m *Metrics) startEndpoint(
 				logrus.Fatalf("Creating key pair reloader: %v", reloadErr)
 			}
 
-			srv := http.Server{
-				Handler: me,
-				TLSConfig: &tls.Config{
-					GetCertificate: kpr.getCertificate,
-					MinVersion:     tls.VersionTLS12,
-				},
+			srv.TLSConfig = &tls.Config{
+				GetCertificate: kpr.getCertificate,
+				MinVersion:     tls.VersionTLS12,
 			}
+
+			go func() {
+				<-stop
+				if err := srv.Shutdown(context.Background()); err != nil {
+					logrus.Errorf("Error on metrics server shutdown: %v", err)
+				}
+			}()
 			err = srv.ServeTLS(l, m.config.MetricsCert, m.config.MetricsKey)
 		} else {
 			logrus.Infof("Serving metrics on %s via HTTP", address)
-			err = http.Serve(l, me)
+			go func() {
+				<-stop
+				if err := srv.Shutdown(context.Background()); err != nil {
+					logrus.Errorf("Error on metrics server shutdown: %v", err)
+				}
+			}()
+			err = srv.Serve(l)
 		}
 
-		if err != nil {
-			logrus.Fatalf("Failed to serve metrics endpoint %v: %v", l, err)
+		if err != nil && !errors.Is(err, http.ErrServerClosed) {
+			logrus.Errorf("Failed to serve metrics endpoint %v: %v", l, err)
 		}
 	}()
 
