@@ -22,9 +22,11 @@ import (
 	sboxfactory "github.com/cri-o/cri-o/internal/factory/sandbox"
 	"github.com/cri-o/cri-o/internal/lib"
 	libsandbox "github.com/cri-o/cri-o/internal/lib/sandbox"
+	"github.com/cri-o/cri-o/internal/linklogs"
 	"github.com/cri-o/cri-o/internal/log"
 	oci "github.com/cri-o/cri-o/internal/oci"
 	"github.com/cri-o/cri-o/internal/resourcestore"
+	"github.com/cri-o/cri-o/internal/runtimehandlerhooks"
 	ann "github.com/cri-o/cri-o/pkg/annotations"
 	libconfig "github.com/cri-o/cri-o/pkg/config"
 	"github.com/cri-o/cri-o/utils"
@@ -302,7 +304,10 @@ func convertToStorageIDMap(mappings []*types.IDMapping) []idtools.IDMap {
 	return ret
 }
 
-func (s *Server) getSandboxIDMappings(sb *libsandbox.Sandbox) (*idtools.IDMappings, error) {
+func (s *Server) getSandboxIDMappings(ctx context.Context, sb *libsandbox.Sandbox) (*idtools.IDMappings, error) {
+	_, span := log.StartSpan(ctx)
+	defer span.End()
+
 	ic := sb.InfraContainer()
 	if ic != nil {
 		mappings := ic.IDMappings()
@@ -333,6 +338,8 @@ func (s *Server) getSandboxIDMappings(sb *libsandbox.Sandbox) (*idtools.IDMappin
 }
 
 func (s *Server) runPodSandbox(ctx context.Context, req *types.RunPodSandboxRequest) (resp *types.RunPodSandboxResponse, retErr error) {
+	ctx, span := log.StartSpan(ctx)
+	defer span.End()
 	sbox := sboxfactory.New()
 	if err := sbox.SetConfig(req.Config); err != nil {
 		return nil, fmt.Errorf("setting sandbox config: %w", err)
@@ -340,13 +347,16 @@ func (s *Server) runPodSandbox(ctx context.Context, req *types.RunPodSandboxRequ
 
 	pathsToChown := []string{}
 
-	// we need to fill in the container name, as it is not present in the request. Luckily, it is a constant.
-	log.Infof(ctx, "Running pod sandbox: %s%s", translateLabelsToDescription(sbox.Config().Labels), oci.InfraContainerName)
-
 	kubeName := sbox.Config().Metadata.Name
+	kubePodUID := sbox.Config().Metadata.Uid
 	namespace := sbox.Config().Metadata.Namespace
 	attempt := sbox.Config().Metadata.Attempt
 
+	// These fields are populated by the Kubelet, but not crictl. Populate if needed.
+	sbox.Config().Labels = populateSandboxLabels(sbox.Config().Labels, kubeName, kubePodUID, namespace)
+	// we need to fill in the container name, as it is not present in the request. Luckily, it is a constant.
+	log.Infof(ctx, "Running pod sandbox: %s%s", translateLabelsToDescription(sbox.Config().Labels), oci.InfraContainerName)
+
 	if err := sbox.SetNameAndID(); err != nil {
 		return nil, fmt.Errorf("setting pod sandbox name and id: %w", err)
 	}
@@ -383,7 +393,7 @@ func (s *Server) runPodSandbox(ctx context.Context, req *types.RunPodSandboxRequ
 		return nil
 	})
 
-	s.resourceStore.SetStageForResource(sbox.Name(), "sandbox creating")
+	s.resourceStore.SetStageForResource(ctx, sbox.Name(), "sandbox creating")
 
 	securityContext := sbox.Config().Linux.SecurityContext
 
@@ -402,7 +412,7 @@ func (s *Server) runPodSandbox(ctx context.Context, req *types.RunPodSandboxRequ
 		}
 		log.Infof(ctx, "CNI plugin is now ready. Continuing to create %s", sbox.Name())
 	}
-	s.resourceStore.SetStageForResource(sbox.Name(), "sandbox network ready")
+	s.resourceStore.SetStageForResource(ctx, sbox.Name(), "sandbox network ready")
 
 	// validate the runtime handler
 	runtimeHandler, err := s.runtimeHandler(req)
@@ -428,7 +438,7 @@ func (s *Server) runPodSandbox(ctx context.Context, req *types.RunPodSandboxRequ
 		return nil, err
 	}
 	resourceCleaner.Add(ctx, "runSandbox: releasing container name: "+containerName, func() error {
-		s.ReleaseContainerName(containerName)
+		s.ReleaseContainerName(ctx, containerName)
 		return nil
 	})
 
@@ -440,7 +450,7 @@ func (s *Server) runPodSandbox(ctx context.Context, req *types.RunPodSandboxRequ
 
 	privileged := s.privilegedSandbox(req)
 
-	s.resourceStore.SetStageForResource(sbox.Name(), "sandbox storage creation")
+	s.resourceStore.SetStageForResource(ctx, sbox.Name(), "sandbox storage creation")
 	podContainer, err := s.StorageRuntimeServer().CreatePodSandbox(s.config.SystemContext,
 		sbox.Name(), sbox.ID(),
 		s.config.PauseImage,
@@ -462,7 +472,7 @@ func (s *Server) runPodSandbox(ctx context.Context, req *types.RunPodSandboxRequ
 		return nil, fmt.Errorf("creating pod sandbox with name %q: %w", sbox.Name(), err)
 	}
 	resourceCleaner.Add(ctx, "runSandbox: removing pod sandbox from storage: "+sbox.ID(), func() error {
-		return s.StorageRuntimeServer().DeleteContainer(sbox.ID())
+		return s.StorageRuntimeServer().DeleteContainer(ctx, sbox.ID())
 	})
 
 	mountLabel := podContainer.MountLabel
@@ -541,7 +551,7 @@ func (s *Server) runPodSandbox(ctx context.Context, req *types.RunPodSandboxRequ
 	g.RemoveMount(libsandbox.DevShmPath)
 
 	// create shm mount for the pod containers.
-	s.resourceStore.SetStageForResource(sbox.Name(), "sandbox shm creation")
+	s.resourceStore.SetStageForResource(ctx, sbox.Name(), "sandbox shm creation")
 	var shmPath string
 	if hostIPC {
 		shmPath = libsandbox.DevShmPath
@@ -554,7 +564,7 @@ func (s *Server) runPodSandbox(ctx context.Context, req *types.RunPodSandboxRequ
 			}
 			shmSize = quantity.Value()
 		}
-		shmPath, err = setupShm(podContainer.RunDir, mountLabel, shmSize)
+		shmPath, err = setupShm(ctx, podContainer.RunDir, mountLabel, shmSize)
 		if err != nil {
 			return nil, err
 		}
@@ -567,7 +577,14 @@ func (s *Server) runPodSandbox(ctx context.Context, req *types.RunPodSandboxRequ
 		})
 	}
 
-	s.resourceStore.SetStageForResource(sbox.Name(), "sandbox spec configuration")
+	// Link logs if requested
+	if emptyDirVolName, ok := kubeAnnotations[ann.LinkLogsAnnotation]; ok {
+		if err = linklogs.MountPodLogs(ctx, kubePodUID, emptyDirVolName, namespace, kubeName, mountLabel); err != nil {
+			log.Warnf(ctx, "Failed to link logs: %v", err)
+		}
+	}
+
+	s.resourceStore.SetStageForResource(ctx, sbox.Name(), "sandbox spec configuration")
 
 	mnt := spec.Mount{
 		Type:        "bind",
@@ -578,7 +595,7 @@ func (s *Server) runPodSandbox(ctx context.Context, req *types.RunPodSandboxRequ
 	// bind mount the pod shm
 	g.AddMount(mnt)
 
-	err = s.setPodSandboxMountLabel(sbox.ID(), mountLabel)
+	err = s.setPodSandboxMountLabel(ctx, sbox.ID(), mountLabel)
 	if err != nil {
 		return nil, err
 	}
@@ -676,11 +693,11 @@ func (s *Server) runPodSandbox(ctx context.Context, req *types.RunPodSandboxRequ
 
 	sb.SetDNSConfig(sbox.Config().DnsConfig)
 
-	if err := s.addSandbox(sb); err != nil {
+	if err := s.addSandbox(ctx, sb); err != nil {
 		return nil, err
 	}
 	resourceCleaner.Add(ctx, "runSandbox: removing pod sandbox "+sbox.ID(), func() error {
-		if err := s.removeSandbox(sbox.ID()); err != nil {
+		if err := s.removeSandbox(ctx, sbox.ID()); err != nil {
 			return fmt.Errorf("could not remove pod sandbox: %w", err)
 		}
 		return nil
@@ -707,8 +724,8 @@ func (s *Server) runPodSandbox(ctx context.Context, req *types.RunPodSandboxRequ
 	sysctls := s.configureGeneratorForSysctls(ctx, g, hostNetwork, hostIPC, req.Config.Linux.Sysctls)
 
 	// set up namespaces
-	s.resourceStore.SetStageForResource(sbox.Name(), "sandbox namespace creation")
-	nsCleanupFuncs, err := s.configureGeneratorForSandboxNamespaces(hostNetwork, hostIPC, hostPID, sandboxIDMappings, sysctls, sb, g)
+	s.resourceStore.SetStageForResource(ctx, sbox.Name(), "sandbox namespace creation")
+	nsCleanupFuncs, err := s.configureGeneratorForSandboxNamespaces(ctx, hostNetwork, hostIPC, hostPID, sandboxIDMappings, sysctls, sb, g)
 	// We want to cleanup after ourselves if we are managing any namespaces and fail in this function.
 	// However, we don't immediately register this func with resourceCleaner because we need to pair the
 	// ns cleanup with networkStop. Otherwise, we could try to cleanup the namespace before the network stop runs,
@@ -731,7 +748,7 @@ func (s *Server) runPodSandbox(ctx context.Context, req *types.RunPodSandboxRequ
 	var ips []string
 	var result cnitypes.Result
 
-	s.resourceStore.SetStageForResource(sbox.Name(), "sandbox network creation")
+	s.resourceStore.SetStageForResource(ctx, sbox.Name(), "sandbox network creation")
 	ips, result, err = s.networkStart(ctx, sb)
 	if err != nil {
 		resourceCleaner.Add(ctx, nsCleanupDescription, nsCleanupFunc)
@@ -758,14 +775,14 @@ func (s *Server) runPodSandbox(ctx context.Context, req *types.RunPodSandboxRequ
 		}
 		g.AddAnnotation(annotations.CNIResult, string(cniResultJSON))
 	}
-	s.resourceStore.SetStageForResource(sbox.Name(), "sandbox storage start")
+	s.resourceStore.SetStageForResource(ctx, sbox.Name(), "sandbox storage start")
 
 	mountPoint, err := s.StorageRuntimeServer().StartContainer(sbox.ID())
 	if err != nil {
 		return nil, fmt.Errorf("failed to mount container %s in pod sandbox %s(%s): %w", containerName, sb.Name(), sbox.ID(), err)
 	}
 	resourceCleaner.Add(ctx, "runSandbox: stopping storage container for sandbox "+sbox.ID(), func() error {
-		if err := s.StorageRuntimeServer().StopContainer(sbox.ID()); err != nil {
+		if err := s.StorageRuntimeServer().StopContainer(ctx, sbox.ID()); err != nil {
 			return fmt.Errorf("could not stop storage container: %s: %w", sbox.ID(), err)
 		}
 		return nil
@@ -855,8 +872,14 @@ func (s *Server) runPodSandbox(ctx context.Context, req *types.RunPodSandboxRequ
 	g.AddAnnotation(annotations.SeccompProfilePath, seccompProfilePath)
 	sb.SetSeccompProfilePath(seccompProfilePath)
 	if !privileged {
-		if err := s.Config().Seccomp().Setup(
-			ctx, g, securityContext.Seccomp, seccompProfilePath,
+		if _, err := s.config.Seccomp().Setup(
+			ctx,
+			nil,
+			"",
+			nil,
+			g,
+			securityContext.Seccomp,
+			seccompProfilePath,
 		); err != nil {
 			return nil, fmt.Errorf("setup seccomp: %w", err)
 		}
@@ -907,7 +930,7 @@ func (s *Server) runPodSandbox(ctx context.Context, req *types.RunPodSandboxRequ
 		return nil, err
 	}
 
-	if err := sb.SetContainerEnvFile(); err != nil {
+	if err := sb.SetContainerEnvFile(ctx); err != nil {
 		return nil, err
 	}
 
@@ -918,9 +941,9 @@ func (s *Server) runPodSandbox(ctx context.Context, req *types.RunPodSandboxRequ
 		return nil, fmt.Errorf("failed to write runtime configuration for pod sandbox %s(%s): %w", sb.Name(), sbox.ID(), err)
 	}
 
-	s.addInfraContainer(container)
+	s.addInfraContainer(ctx, container)
 	resourceCleaner.Add(ctx, "runSandbox: removing infra container "+container.ID(), func() error {
-		s.removeInfraContainer(container)
+		s.removeInfraContainer(ctx, container)
 		return nil
 	})
 
@@ -936,11 +959,21 @@ func (s *Server) runPodSandbox(ctx context.Context, req *types.RunPodSandboxRequ
 		}
 	}
 
-	s.resourceStore.SetStageForResource(sbox.Name(), "sandbox container runtime creation")
+	s.resourceStore.SetStageForResource(ctx, sbox.Name(), "sandbox container runtime creation")
 	if err := s.createContainerPlatform(ctx, container, sb.CgroupParent(), sandboxIDMappings); err != nil {
 		return nil, err
 	}
 
+	hooks, err := runtimehandlerhooks.GetRuntimeHandlerHooks(ctx, &s.config, sb.RuntimeHandler(), sb.Annotations())
+	if err != nil {
+		return nil, fmt.Errorf("failed to get runtime handler %q hooks", sb.RuntimeHandler())
+	}
+	if hooks != nil {
+		if err := hooks.PreStart(ctx, container, sb); err != nil {
+			return nil, fmt.Errorf("failed to run pre-stop hook for container %q: %w", sb.ID(), err)
+		}
+	}
+	s.generateCRIEvent(ctx, sb.InfraContainer(), types.ContainerEventType_CONTAINER_CREATED_EVENT)
 	if err := s.Runtime().StartContainer(ctx, container); err != nil {
 		return nil, err
 	}
@@ -970,6 +1003,10 @@ func (s *Server) runPodSandbox(ctx context.Context, req *types.RunPodSandboxRequ
 	}
 	sb.AddIPs(ips)
 
+	if err := s.nri.runPodSandbox(ctx, sb); err != nil {
+		return nil, err
+	}
+
 	if isContextError(ctx.Err()) {
 		if err := s.resourceStore.Put(sbox.Name(), sb, resourceCleaner); err != nil {
 			log.Errorf(ctx, "RunSandbox: failed to save progress of sandbox %s: %v", sbox.ID(), err)
@@ -977,14 +1014,40 @@ func (s *Server) runPodSandbox(ctx context.Context, req *types.RunPodSandboxRequ
 		log.Infof(ctx, "RunSandbox: context was either canceled or the deadline was exceeded: %v", ctx.Err())
 		return nil, ctx.Err()
 	}
+
+	// Since it's not a context error, we can delete the resource from the store, it will be tracked in the server from now on.
+	s.resourceStore.Delete(sbox.Name())
+
 	sb.SetCreated()
+	s.generateCRIEvent(ctx, sb.InfraContainer(), types.ContainerEventType_CONTAINER_STARTED_EVENT)
 
 	log.Infof(ctx, "Ran pod sandbox %s with infra container: %s", container.ID(), container.Description())
 	resp = &types.RunPodSandboxResponse{PodSandboxId: sbox.ID()}
 	return resp, nil
 }
 
-func setupShm(podSandboxRunDir, mountLabel string, shmSize int64) (shmPath string, _ error) {
+// populateSandboxLabels adds some fields that Kubelet specifies by default, but other clients (crictl) does not.
+// While CRI-O typically only cares about the kubelet, the cost here is low. Adding this code prevents issues
+// with the LogLink feature, as the unmounting relies on the existence of the UID in the sandbox labels.
+func populateSandboxLabels(labels map[string]string, kubeName, kubePodUID, namespace string) map[string]string {
+	if labels == nil {
+		labels = make(map[string]string)
+	}
+	if _, ok := labels[kubeletTypes.KubernetesPodNameLabel]; !ok {
+		labels[kubeletTypes.KubernetesPodNameLabel] = kubeName
+	}
+	if _, ok := labels[kubeletTypes.KubernetesPodNamespaceLabel]; !ok {
+		labels[kubeletTypes.KubernetesPodNamespaceLabel] = namespace
+	}
+	if _, ok := labels[kubeletTypes.KubernetesPodUIDLabel]; !ok {
+		labels[kubeletTypes.KubernetesPodUIDLabel] = kubePodUID
+	}
+	return labels
+}
+
+func setupShm(ctx context.Context, podSandboxRunDir, mountLabel string, shmSize int64) (shmPath string, _ error) {
+	_, span := log.StartSpan(ctx)
+	defer span.End()
 	if shmSize <= 0 {
 		return "", fmt.Errorf("shm size %d must be greater than 0", shmSize)
 	}
@@ -1002,6 +1065,8 @@ func setupShm(podSandboxRunDir, mountLabel string, shmSize int64) (shmPath strin
 }
 
 func (s *Server) configureGeneratorForSysctls(ctx context.Context, g *generate.Generator, hostNetwork, hostIPC bool, sysctls map[string]string) map[string]string {
+	ctx, span := log.StartSpan(ctx)
+	defer span.End()
 	sysctlsToReturn := make(map[string]string)
 	defaultSysctls, err := s.config.RuntimeConfig.Sysctls()
 	if err != nil {
@@ -1035,7 +1100,9 @@ func (s *Server) configureGeneratorForSysctls(ctx context.Context, g *generate.G
 // as well as whether CRI-O should be managing the namespace lifecycle.
 // it returns a slice of cleanup funcs, all of which are the respective NamespaceRemove() for the sandbox.
 // The caller should defer the cleanup funcs if there is an error, to make sure each namespace we are managing is properly cleaned up.
-func (s *Server) configureGeneratorForSandboxNamespaces(hostNetwork, hostIPC, hostPID bool, idMappings *idtools.IDMappings, sysctls map[string]string, sb *libsandbox.Sandbox, g *generate.Generator) (cleanupFuncs []func() error, retErr error) {
+func (s *Server) configureGeneratorForSandboxNamespaces(ctx context.Context, hostNetwork, hostIPC, hostPID bool, idMappings *idtools.IDMappings, sysctls map[string]string, sb *libsandbox.Sandbox, g *generate.Generator) (cleanupFuncs []func() error, retErr error) {
+	_, span := log.StartSpan(ctx)
+	defer span.End()
 	// Since we need a process to hold open the PID namespace, CRI-O can't manage the NS lifecycle
 	if hostPID {
 		if err := g.RemoveLinuxNamespace(string(spec.PIDNamespace)); err != nil {
