@@ -1,3 +1,4 @@
+//go:build !linux
 // +build !linux
 
 package server
@@ -6,7 +7,7 @@ import (
 	"context"
 	"fmt"
 
-	"github.com/cri-o/cri-o/server/cri/types"
+	types "k8s.io/cri-api/pkg/apis/runtime/v1"
 )
 
 func (s *Server) runPodSandbox(ctx context.Context, req *types.RunPodSandboxRequest) (*types.RunPodSandboxResponse, error) {
