@@ -6,6 +6,7 @@ import (
 	"crypto/x509"
 	"errors"
 	"fmt"
+	"io/fs"
 	"net"
 	"net/http"
 	"os"
@@ -16,9 +17,15 @@ import (
 	"sync"
 	"time"
 
+	kubetypes "k8s.io/kubernetes/pkg/kubelet/types"
+
+	"google.golang.org/grpc/codes"
+	"google.golang.org/grpc/status"
+
 	imageTypes "github.com/containers/image/v5/types"
 	"github.com/containers/storage/pkg/idtools"
 	storageTypes "github.com/containers/storage/types"
+	"github.com/cri-o/cri-o/internal/config/seccomp"
 	"github.com/cri-o/cri-o/internal/hostport"
 	"github.com/cri-o/cri-o/internal/lib"
 	"github.com/cri-o/cri-o/internal/lib/sandbox"
@@ -33,9 +40,12 @@ import (
 	"github.com/cri-o/cri-o/server/streaming"
 	"github.com/cri-o/cri-o/utils"
 	"github.com/fsnotify/fsnotify"
+	"github.com/opencontainers/runtime-spec/specs-go"
 	"github.com/sirupsen/logrus"
 	"golang.org/x/sys/unix"
 	types "k8s.io/cri-api/pkg/apis/runtime/v1"
+
+	nriIf "github.com/cri-o/cri-o/internal/nri"
 )
 
 const (
@@ -61,8 +71,9 @@ type Server struct {
 	hostportManager hostport.HostPortManager
 
 	*lib.ContainerServer
-	monitorsChan      chan struct{}
-	defaultIDMappings *idtools.IDMappings
+	monitorsChan        chan struct{}
+	defaultIDMappings   *idtools.IDMappings
+	ContainerEventsChan chan types.ContainerEventResponse
 
 	minimumMappableUID, minimumMappableGID int64
 
@@ -73,6 +84,12 @@ type Server struct {
 	pullOperationsLock sync.Mutex
 
 	resourceStore *resourcestore.ResourceStore
+
+	seccompNotifierChan chan seccomp.Notification
+	seccompNotifiers    sync.Map
+
+	// NRI runtime interface
+	nri *nriAPI
 }
 
 // pullArguments are used to identify a pullOperation via an input image name and
@@ -163,6 +180,8 @@ func (s *Server) getPortForward(req *types.PortForwardRequest) (*types.PortForwa
 // For every container it fails to restore, it returns that containers image, so that
 // it can be cleaned up (if we're using internal_wipe).
 func (s *Server) restore(ctx context.Context) []string {
+	ctx, span := log.StartSpan(ctx)
+	defer span.End()
 	containersAndTheirImages := map[string]string{}
 	containers, err := s.Store().Containers()
 	if err != nil && !errors.Is(err, os.ErrNotExist) {
@@ -205,7 +224,7 @@ func (s *Server) restore(ctx context.Context) []string {
 			}
 			// Release the infra container name and the pod name for future use
 			if strings.Contains(n, oci.InfraContainerName) {
-				s.ReleaseContainerName(n)
+				s.ReleaseContainerName(ctx, n)
 			} else {
 				s.ReleasePodName(n)
 			}
@@ -221,7 +240,7 @@ func (s *Server) restore(ctx context.Context) []string {
 					log.Warnf(ctx, "Unable to delete container %s: %v", n, err)
 				}
 				// Release the container name for future use
-				s.ReleaseContainerName(n)
+				s.ReleaseContainerName(ctx, n)
 			}
 			// Remove the container from the list of podContainers, or else we'll retry the delete later,
 			// causing a useless debug message.
@@ -248,7 +267,7 @@ func (s *Server) restore(ctx context.Context) []string {
 				log.Warnf(ctx, "Unable to delete container %s: %v", n, err)
 			}
 			// Release the container name
-			s.ReleaseContainerName(n)
+			s.ReleaseContainerName(ctx, n)
 		}
 	}
 
@@ -276,7 +295,7 @@ func (s *Server) restore(ctx context.Context) []string {
 
 	// Restore sandbox IPs
 	for _, sb := range s.ListSandboxes() {
-		ips, err := s.getSandboxIPs(sb)
+		ips, err := s.getSandboxIPs(ctx, sb)
 		if err != nil {
 			log.Warnf(ctx, "Could not restore sandbox IP for %v: %v", sb.ID(), err)
 			continue
@@ -330,6 +349,11 @@ func (s *Server) Shutdown(ctx context.Context) error {
 		}
 	}
 
+	if s.config.EnablePodEvents {
+		// closing a non-nil channel only if the evented pleg is enabled
+		close(s.ContainerEventsChan)
+	}
+
 	return nil
 }
 
@@ -424,7 +448,10 @@ func New(
 		pullOperationsInProgress: make(map[pullArguments]*pullOperation),
 		resourceStore:            resourcestore.New(),
 	}
-
+	if s.config.EnablePodEvents {
+		// creating a container events channel only if the evented pleg is enabled
+		s.ContainerEventsChan = make(chan types.ContainerEventResponse, 1000)
+	}
 	if err := configureMaxThreads(); err != nil {
 		return nil, err
 	}
@@ -512,6 +539,25 @@ func New(
 		logrus.Debug("Metrics are disabled")
 	}
 
+	if err := s.startSeccompNotifierWatcher(ctx); err != nil {
+		return nil, fmt.Errorf("start seccomp notifier watcher: %w", err)
+	}
+
+	// Set up our NRI adaptation.
+	api, err := nriIf.New(s.Config().NRI)
+	if err != nil {
+		return nil, fmt.Errorf("failed to create NRI interface: %v", err)
+	}
+
+	s.nri = &nriAPI{
+		cri: s,
+		nri: api,
+	}
+
+	if err := s.nri.start(); err != nil {
+		return nil, err
+	}
+
 	return s, nil
 }
 
@@ -530,6 +576,8 @@ func useDefaultUmask() {
 // indicates an upgrade has happened, it attempts to wipe that list of images.
 // This attempt is best-effort.
 func (s *Server) wipeIfAppropriate(ctx context.Context, imagesToDelete []string) {
+	ctx, span := log.StartSpan(ctx)
+	defer span.End()
 	if !s.config.InternalWipe {
 		return
 	}
@@ -590,39 +638,57 @@ func (s *Server) wipeIfAppropriate(ctx context.Context, imagesToDelete []string)
 	}
 }
 
-func (s *Server) addSandbox(sb *sandbox.Sandbox) error {
-	return s.ContainerServer.AddSandbox(sb)
+func (s *Server) addSandbox(ctx context.Context, sb *sandbox.Sandbox) error {
+	ctx, span := log.StartSpan(ctx)
+	defer span.End()
+	return s.ContainerServer.AddSandbox(ctx, sb)
 }
 
-func (s *Server) getSandbox(id string) *sandbox.Sandbox {
+func (s *Server) getSandbox(ctx context.Context, id string) *sandbox.Sandbox {
+	_, span := log.StartSpan(ctx)
+	defer span.End()
 	return s.ContainerServer.GetSandbox(id)
 }
 
-func (s *Server) removeSandbox(id string) error {
-	return s.ContainerServer.RemoveSandbox(id)
+func (s *Server) removeSandbox(ctx context.Context, id string) error {
+	ctx, span := log.StartSpan(ctx)
+	defer span.End()
+	return s.ContainerServer.RemoveSandbox(ctx, id)
 }
 
-func (s *Server) addContainer(c *oci.Container) {
-	s.ContainerServer.AddContainer(c)
+func (s *Server) addContainer(ctx context.Context, c *oci.Container) {
+	ctx, span := log.StartSpan(ctx)
+	defer span.End()
+	s.ContainerServer.AddContainer(ctx, c)
 }
 
-func (s *Server) addInfraContainer(c *oci.Container) {
-	s.ContainerServer.AddInfraContainer(c)
+func (s *Server) addInfraContainer(ctx context.Context, c *oci.Container) {
+	ctx, span := log.StartSpan(ctx)
+	defer span.End()
+	s.ContainerServer.AddInfraContainer(ctx, c)
 }
 
-func (s *Server) getInfraContainer(id string) *oci.Container {
-	return s.ContainerServer.GetInfraContainer(id)
+func (s *Server) getInfraContainer(ctx context.Context, id string) *oci.Container {
+	ctx, span := log.StartSpan(ctx)
+	defer span.End()
+	return s.ContainerServer.GetInfraContainer(ctx, id)
 }
 
-func (s *Server) removeContainer(c *oci.Container) {
-	s.ContainerServer.RemoveContainer(c)
+func (s *Server) removeContainer(ctx context.Context, c *oci.Container) {
+	ctx, span := log.StartSpan(ctx)
+	defer span.End()
+	s.ContainerServer.RemoveContainer(ctx, c)
 }
 
-func (s *Server) removeInfraContainer(c *oci.Container) {
-	s.ContainerServer.RemoveInfraContainer(c)
+func (s *Server) removeInfraContainer(ctx context.Context, c *oci.Container) {
+	ctx, span := log.StartSpan(ctx)
+	defer span.End()
+	s.ContainerServer.RemoveInfraContainer(ctx, c)
 }
 
-func (s *Server) getPodSandboxFromRequest(podSandboxID string) (*sandbox.Sandbox, error) {
+func (s *Server) getPodSandboxFromRequest(ctx context.Context, podSandboxID string) (*sandbox.Sandbox, error) {
+	ctx, span := log.StartSpan(ctx)
+	defer span.End()
 	if podSandboxID == "" {
 		return nil, sandbox.ErrIDEmpty
 	}
@@ -632,7 +698,7 @@ func (s *Server) getPodSandboxFromRequest(podSandboxID string) (*sandbox.Sandbox
 		return nil, fmt.Errorf("PodSandbox with ID starting with %s not found: %w", podSandboxID, err)
 	}
 
-	sb := s.getSandbox(sandboxID)
+	sb := s.getSandbox(ctx, sandboxID)
 	if sb == nil {
 		return nil, fmt.Errorf("specified pod sandbox not found: %s", sandboxID)
 	}
@@ -652,6 +718,111 @@ func (s *Server) MonitorsCloseChan() chan struct{} {
 	return s.monitorsChan
 }
 
+func (s *Server) startSeccompNotifierWatcher(ctx context.Context) error {
+	logrus.Info("Starting seccomp notifier watcher")
+	s.seccompNotifierChan = make(chan seccomp.Notification)
+
+	// Restore or cleanup
+	notifierPath := s.config.Seccomp().NotifierPath()
+	info, err := os.Stat(notifierPath)
+	if err == nil && info.IsDir() {
+		if err := filepath.Walk(notifierPath, func(path string, info fs.FileInfo, err error) error {
+			if err != nil {
+				return err
+			}
+			if info.IsDir() {
+				return nil
+			}
+
+			id := info.Name()
+
+			if err := os.RemoveAll(path); err != nil {
+				logrus.Error("Unable to remove path: %w", err)
+				return nil
+			}
+
+			ctr, err := s.ContainerServer.GetContainerFromShortID(ctx, id)
+			if err != nil {
+				logrus.Warnf("Skipping not existing seccomp notifier container ID: %s", id)
+				return nil
+			}
+
+			if ctr.State().Status != specs.StateRunning {
+				logrus.Warnf("Skipping container %s because it is not running any more", id)
+				return nil
+			}
+
+			// Restart the notifier
+			notifier, err := seccomp.NewNotifier(context.Background(), s.seccompNotifierChan, id, path, ctr.Annotations())
+			if err != nil {
+				logrus.Errorf("Unable to run restored notifier: %v", err)
+				return nil
+			}
+
+			s.seccompNotifiers.Store(id, notifier)
+
+			return nil
+		}); err != nil {
+			return fmt.Errorf("unable to walk seccomp listener dir: %w", err)
+		}
+	} else {
+		if err := os.RemoveAll(notifierPath); err != nil {
+			return fmt.Errorf("unable to remove default seccomp listener dir: %w", err)
+		}
+
+		if err := os.MkdirAll(notifierPath, 0o700); err != nil {
+			return fmt.Errorf("unable to create default seccomp listener dir: %w", err)
+		}
+	}
+
+	// Start the notifier watcher
+	go func() {
+		for {
+			msg := <-s.seccompNotifierChan
+			ctx := msg.Ctx()
+			id := msg.ContainerID()
+			syscall := msg.Syscall()
+
+			log.Infof(ctx, "Got seccomp notifier message for container ID: %s (syscall = %s)", id, syscall)
+
+			result, ok := s.seccompNotifiers.Load(id)
+			if !ok {
+				log.Errorf(ctx, "Unable to get notifier for container ID")
+				continue
+			}
+			notifier, ok := result.(*seccomp.Notifier)
+			if !ok {
+				log.Errorf(ctx, "Notifier is not a seccomp notifier type")
+				continue
+			}
+			notifier.AddSyscall(syscall)
+
+			ctr := s.ContainerServer.GetContainer(ctx, id)
+			usedSyscalls := notifier.UsedSyscalls()
+
+			if notifier.StopContainers() {
+				// Stop the container only if the notifier timer has expired
+				// The timer will be refreshed after each call to OnExpired.
+				notifier.OnExpired(func() {
+					log.Infof(ctx, "Seccomp notifier timer expired, stopping container %s", id)
+
+					state := ctr.StateNoLock()
+					state.SeccompKilled = true
+					state.Error = "Used forbidden syscalls: " + usedSyscalls
+
+					if err := s.stopContainer(context.Background(), ctr, 0); err != nil {
+						log.Errorf(ctx, "Unable to stop container %s: %v", id, err)
+					}
+				})
+			}
+
+			metrics.Instance().MetricContainersSeccompNotifierCountTotalInc(ctr.Name(), syscall)
+		}
+	}()
+
+	return nil
+}
+
 // StartExitMonitor start a routine that monitors container exits
 // and updates the container status
 func (s *Server) StartExitMonitor(ctx context.Context) {
@@ -677,6 +848,9 @@ func (s *Server) monitorExits(ctx context.Context, watcher *fsnotify.Watcher, do
 			go s.handleExit(ctx, event)
 		case err := <-watcher.Errors:
 			log.Debugf(ctx, "Watch error: %v", err)
+			if s.config.EnablePodEvents {
+				close(s.ContainerEventsChan)
+			}
 			close(done)
 			return
 		case <-s.monitorsChan:
@@ -688,21 +862,27 @@ func (s *Server) monitorExits(ctx context.Context, watcher *fsnotify.Watcher, do
 }
 
 func (s *Server) handleExit(ctx context.Context, event fsnotify.Event) {
+	ctx, span := log.StartSpan(ctx)
+	defer span.End()
 	log.Debugf(ctx, "Event: %v", event)
 	if event.Op&fsnotify.Create != fsnotify.Create {
 		return
 	}
 	containerID := filepath.Base(event.Name)
 	log.Debugf(ctx, "Container or sandbox exited: %v", containerID)
-	c := s.GetContainer(containerID)
+	c := s.GetContainer(ctx, containerID)
+	nriCtr := c
 	resource := "container"
+	var sb *sandbox.Sandbox
 	if c == nil {
-		sb := s.GetSandbox(containerID)
+		sb = s.GetSandbox(containerID)
 		if sb == nil {
 			return
 		}
 		c = sb.InfraContainer()
 		resource = "sandbox infra"
+	} else {
+		sb = s.GetSandbox(c.Sandbox())
 	}
 	log.Debugf(ctx, "%s exited and found: %v", resource, containerID)
 	if err := s.Runtime().UpdateContainerStatus(ctx, c); err != nil {
@@ -712,7 +892,137 @@ func (s *Server) handleExit(ctx context.Context, event fsnotify.Event) {
 	if err := s.ContainerStateToDisk(ctx, c); err != nil {
 		log.Warnf(ctx, "Unable to write %s %s state to disk: %v", resource, c.ID(), err)
 	}
+
+	if nriCtr != nil {
+		if err := s.nri.stopContainer(ctx, nil, nriCtr); err != nil {
+			log.Warnf(ctx, "NRI stop container request of %s failed: %v", nriCtr.ID(), err)
+		}
+	}
+
+	hooks, err := runtimehandlerhooks.GetRuntimeHandlerHooks(ctx, &s.config, sb.RuntimeHandler(), sb.Annotations())
+	if err != nil {
+		log.Warnf(ctx, "Failed to get runtime handler %q hooks", sb.RuntimeHandler())
+	} else if hooks != nil {
+		if err := hooks.PostStop(ctx, c, sb); err != nil {
+			log.Errorf(ctx, "Failed to run post-stop hook for container %s: %v", c.ID(), err)
+		}
+	}
+
+	s.generateCRIEvent(ctx, c, types.ContainerEventType_CONTAINER_STOPPED_EVENT)
 	if err := os.Remove(event.Name); err != nil {
 		log.Warnf(ctx, "Failed to remove exit file: %v", err)
 	}
 }
+
+func (s *Server) getSandboxStatuses(ctx context.Context, sandboxID string) (*types.PodSandboxStatus, error) {
+	sandboxStatusRequest := &types.PodSandboxStatusRequest{PodSandboxId: sandboxID}
+	sandboxStatus, err := s.PodSandboxStatus(ctx, sandboxStatusRequest)
+
+	if isNotFound(err) {
+		return nil, err
+	}
+
+	if err != nil {
+		return nil, fmt.Errorf("getSandboxStatuses: %w", err)
+	}
+
+	return sandboxStatus.GetStatus(), nil
+}
+
+func (s *Server) getContainerStatuses(ctx context.Context, sandboxUID string) ([]*types.ContainerStatus, error) {
+	listContainerRequest := &types.ListContainersRequest{Filter: &types.ContainerFilter{LabelSelector: map[string]string{kubetypes.KubernetesPodUIDLabel: sandboxUID}}}
+	containers, err := s.ListContainers(ctx, listContainerRequest)
+	if err != nil {
+		return []*types.ContainerStatus{}, err
+	}
+
+	containerStatuses := []*types.ContainerStatus{}
+	for _, cc := range containers.GetContainers() {
+		containerStatusRequest := &types.ContainerStatusRequest{ContainerId: cc.Id}
+		resp, err := s.ContainerStatus(ctx, containerStatusRequest)
+		if isNotFound(err) {
+			continue
+		}
+		if err != nil {
+			return []*types.ContainerStatus{}, err
+		}
+		containerStatuses = append(containerStatuses, resp.GetStatus())
+	}
+
+	return containerStatuses, nil
+}
+
+func (s *Server) getContainerStatusesFromSandboxID(ctx context.Context, sandboxID string) ([]*types.ContainerStatus, error) {
+	listContainerRequest := &types.ListContainersRequest{Filter: &types.ContainerFilter{PodSandboxId: sandboxID}}
+	containers, err := s.ListContainers(ctx, listContainerRequest)
+	if err != nil {
+		return []*types.ContainerStatus{}, err
+	}
+
+	containerStatuses := []*types.ContainerStatus{}
+	for _, cc := range containers.GetContainers() {
+		containerStatusRequest := &types.ContainerStatusRequest{ContainerId: cc.Id, Verbose: false}
+		resp, err := s.ContainerStatus(ctx, containerStatusRequest)
+		if isNotFound(err) {
+			continue
+		}
+		if err != nil {
+			return []*types.ContainerStatus{}, err
+		}
+		containerStatuses = append(containerStatuses, resp.GetStatus())
+	}
+
+	return containerStatuses, nil
+}
+
+func (s *Server) generateCRIEvent(ctx context.Context, container *oci.Container, eventType types.ContainerEventType) {
+	// returning no error if the Evented PLEG feature is not enabled
+	if !s.config.EnablePodEvents {
+		return
+	}
+	if err := s.Runtime().UpdateContainerStatus(ctx, container); err != nil {
+		log.Errorf(ctx, "GenerateCRIEvent: event type: %s, failed to update the container status %s: %v", eventType, container.ID(), err)
+		return
+	}
+
+	if !s.HasSandbox(container.Sandbox()) {
+		return
+	}
+
+	sandboxStatuses, err := s.getSandboxStatuses(ctx, s.GetSandbox(container.Sandbox()).ID())
+
+	if isNotFound(err) {
+		return
+	}
+
+	if err != nil {
+		log.Errorf(ctx, "GenerateCRIEvent: event type: %s, failed to get sandbox statuses of the pod %s: %v", eventType, sandboxStatuses.Metadata.Uid, err)
+		return
+	}
+
+	containerStatuses, err := s.getContainerStatuses(ctx, sandboxStatuses.Metadata.Uid)
+	if err != nil {
+		log.Errorf(ctx, "GenerateCRIEvent: event type: %s, failed to get container statuses of the pod %s: %v", eventType, sandboxStatuses.Metadata.Uid, err)
+		return
+	}
+
+	select {
+	case s.ContainerEventsChan <- types.ContainerEventResponse{ContainerId: container.ID(), ContainerEventType: eventType, CreatedAt: time.Now().UnixNano(), PodSandboxStatus: sandboxStatuses, ContainersStatuses: containerStatuses}:
+		log.Debugf(ctx, "Container event %s generated for %s", eventType, container.ID())
+	default:
+		log.Errorf(ctx, "GenerateCRIEvent: failed to generate event %s for container %s", eventType, container.ID())
+		return
+	}
+}
+
+func isNotFound(err error) bool {
+	s, ok := status.FromError(err)
+	if !ok {
+		return ok
+	}
+	if s.Code() == codes.NotFound {
+		return true
+	}
+
+	return false
+}
